# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

from oracle.v1beta1 import query_pb2 as oracle_dot_v1beta1_dot_query__pb2


class QueryStub(object):
    """Query defines the gRPC querier service.
    """

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.ExchangeRate = channel.unary_unary(
                '/nibiru.oracle.v1beta1.Query/ExchangeRate',
                request_serializer=oracle_dot_v1beta1_dot_query__pb2.QueryExchangeRateRequest.SerializeToString,
                response_deserializer=oracle_dot_v1beta1_dot_query__pb2.QueryExchangeRateResponse.FromString,
                )
        self.ExchangeRates = channel.unary_unary(
                '/nibiru.oracle.v1beta1.Query/ExchangeRates',
                request_serializer=oracle_dot_v1beta1_dot_query__pb2.QueryExchangeRatesRequest.SerializeToString,
                response_deserializer=oracle_dot_v1beta1_dot_query__pb2.QueryExchangeRatesResponse.FromString,
                )
        self.TobinTax = channel.unary_unary(
                '/nibiru.oracle.v1beta1.Query/TobinTax',
                request_serializer=oracle_dot_v1beta1_dot_query__pb2.QueryTobinTaxRequest.SerializeToString,
                response_deserializer=oracle_dot_v1beta1_dot_query__pb2.QueryTobinTaxResponse.FromString,
                )
        self.TobinTaxes = channel.unary_unary(
                '/nibiru.oracle.v1beta1.Query/TobinTaxes',
                request_serializer=oracle_dot_v1beta1_dot_query__pb2.QueryTobinTaxesRequest.SerializeToString,
                response_deserializer=oracle_dot_v1beta1_dot_query__pb2.QueryTobinTaxesResponse.FromString,
                )
        self.Actives = channel.unary_unary(
                '/nibiru.oracle.v1beta1.Query/Actives',
                request_serializer=oracle_dot_v1beta1_dot_query__pb2.QueryActivesRequest.SerializeToString,
                response_deserializer=oracle_dot_v1beta1_dot_query__pb2.QueryActivesResponse.FromString,
                )
        self.VoteTargets = channel.unary_unary(
                '/nibiru.oracle.v1beta1.Query/VoteTargets',
                request_serializer=oracle_dot_v1beta1_dot_query__pb2.QueryVoteTargetsRequest.SerializeToString,
                response_deserializer=oracle_dot_v1beta1_dot_query__pb2.QueryVoteTargetsResponse.FromString,
                )
        self.FeederDelegation = channel.unary_unary(
                '/nibiru.oracle.v1beta1.Query/FeederDelegation',
                request_serializer=oracle_dot_v1beta1_dot_query__pb2.QueryFeederDelegationRequest.SerializeToString,
                response_deserializer=oracle_dot_v1beta1_dot_query__pb2.QueryFeederDelegationResponse.FromString,
                )
        self.MissCounter = channel.unary_unary(
                '/nibiru.oracle.v1beta1.Query/MissCounter',
                request_serializer=oracle_dot_v1beta1_dot_query__pb2.QueryMissCounterRequest.SerializeToString,
                response_deserializer=oracle_dot_v1beta1_dot_query__pb2.QueryMissCounterResponse.FromString,
                )
        self.AggregatePrevote = channel.unary_unary(
                '/nibiru.oracle.v1beta1.Query/AggregatePrevote',
                request_serializer=oracle_dot_v1beta1_dot_query__pb2.QueryAggregatePrevoteRequest.SerializeToString,
                response_deserializer=oracle_dot_v1beta1_dot_query__pb2.QueryAggregatePrevoteResponse.FromString,
                )
        self.AggregatePrevotes = channel.unary_unary(
                '/nibiru.oracle.v1beta1.Query/AggregatePrevotes',
                request_serializer=oracle_dot_v1beta1_dot_query__pb2.QueryAggregatePrevotesRequest.SerializeToString,
                response_deserializer=oracle_dot_v1beta1_dot_query__pb2.QueryAggregatePrevotesResponse.FromString,
                )
        self.AggregateVote = channel.unary_unary(
                '/nibiru.oracle.v1beta1.Query/AggregateVote',
                request_serializer=oracle_dot_v1beta1_dot_query__pb2.QueryAggregateVoteRequest.SerializeToString,
                response_deserializer=oracle_dot_v1beta1_dot_query__pb2.QueryAggregateVoteResponse.FromString,
                )
        self.AggregateVotes = channel.unary_unary(
                '/nibiru.oracle.v1beta1.Query/AggregateVotes',
                request_serializer=oracle_dot_v1beta1_dot_query__pb2.QueryAggregateVotesRequest.SerializeToString,
                response_deserializer=oracle_dot_v1beta1_dot_query__pb2.QueryAggregateVotesResponse.FromString,
                )
        self.Params = channel.unary_unary(
                '/nibiru.oracle.v1beta1.Query/Params',
                request_serializer=oracle_dot_v1beta1_dot_query__pb2.QueryParamsRequest.SerializeToString,
                response_deserializer=oracle_dot_v1beta1_dot_query__pb2.QueryParamsResponse.FromString,
                )


class QueryServicer(object):
    """Query defines the gRPC querier service.
    """

    def ExchangeRate(self, request, context):
        """ExchangeRate returns exchange rate of a pair
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ExchangeRates(self, request, context):
        """ExchangeRates returns exchange rates of all pairs
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def TobinTax(self, request, context):
        """TobinTax returns tobin tax of a pair
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def TobinTaxes(self, request, context):
        """TobinTaxes returns tobin taxes of all pairs
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Actives(self, request, context):
        """Actives returns all active pairs
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def VoteTargets(self, request, context):
        """VoteTargets returns all vote target for pairs
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def FeederDelegation(self, request, context):
        """FeederDelegation returns feeder delegation of a validator
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def MissCounter(self, request, context):
        """MissCounter returns oracle miss counter of a validator
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def AggregatePrevote(self, request, context):
        """AggregatePrevote returns an aggregate prevote of a validator
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def AggregatePrevotes(self, request, context):
        """AggregatePrevotes returns aggregate prevotes of all validators
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def AggregateVote(self, request, context):
        """AggregateVote returns an aggregate vote of a validator
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def AggregateVotes(self, request, context):
        """AggregateVotes returns aggregate votes of all validators
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Params(self, request, context):
        """Params queries all parameters.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_QueryServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'ExchangeRate': grpc.unary_unary_rpc_method_handler(
                    servicer.ExchangeRate,
                    request_deserializer=oracle_dot_v1beta1_dot_query__pb2.QueryExchangeRateRequest.FromString,
                    response_serializer=oracle_dot_v1beta1_dot_query__pb2.QueryExchangeRateResponse.SerializeToString,
            ),
            'ExchangeRates': grpc.unary_unary_rpc_method_handler(
                    servicer.ExchangeRates,
                    request_deserializer=oracle_dot_v1beta1_dot_query__pb2.QueryExchangeRatesRequest.FromString,
                    response_serializer=oracle_dot_v1beta1_dot_query__pb2.QueryExchangeRatesResponse.SerializeToString,
            ),
            'TobinTax': grpc.unary_unary_rpc_method_handler(
                    servicer.TobinTax,
                    request_deserializer=oracle_dot_v1beta1_dot_query__pb2.QueryTobinTaxRequest.FromString,
                    response_serializer=oracle_dot_v1beta1_dot_query__pb2.QueryTobinTaxResponse.SerializeToString,
            ),
            'TobinTaxes': grpc.unary_unary_rpc_method_handler(
                    servicer.TobinTaxes,
                    request_deserializer=oracle_dot_v1beta1_dot_query__pb2.QueryTobinTaxesRequest.FromString,
                    response_serializer=oracle_dot_v1beta1_dot_query__pb2.QueryTobinTaxesResponse.SerializeToString,
            ),
            'Actives': grpc.unary_unary_rpc_method_handler(
                    servicer.Actives,
                    request_deserializer=oracle_dot_v1beta1_dot_query__pb2.QueryActivesRequest.FromString,
                    response_serializer=oracle_dot_v1beta1_dot_query__pb2.QueryActivesResponse.SerializeToString,
            ),
            'VoteTargets': grpc.unary_unary_rpc_method_handler(
                    servicer.VoteTargets,
                    request_deserializer=oracle_dot_v1beta1_dot_query__pb2.QueryVoteTargetsRequest.FromString,
                    response_serializer=oracle_dot_v1beta1_dot_query__pb2.QueryVoteTargetsResponse.SerializeToString,
            ),
            'FeederDelegation': grpc.unary_unary_rpc_method_handler(
                    servicer.FeederDelegation,
                    request_deserializer=oracle_dot_v1beta1_dot_query__pb2.QueryFeederDelegationRequest.FromString,
                    response_serializer=oracle_dot_v1beta1_dot_query__pb2.QueryFeederDelegationResponse.SerializeToString,
            ),
            'MissCounter': grpc.unary_unary_rpc_method_handler(
                    servicer.MissCounter,
                    request_deserializer=oracle_dot_v1beta1_dot_query__pb2.QueryMissCounterRequest.FromString,
                    response_serializer=oracle_dot_v1beta1_dot_query__pb2.QueryMissCounterResponse.SerializeToString,
            ),
            'AggregatePrevote': grpc.unary_unary_rpc_method_handler(
                    servicer.AggregatePrevote,
                    request_deserializer=oracle_dot_v1beta1_dot_query__pb2.QueryAggregatePrevoteRequest.FromString,
                    response_serializer=oracle_dot_v1beta1_dot_query__pb2.QueryAggregatePrevoteResponse.SerializeToString,
            ),
            'AggregatePrevotes': grpc.unary_unary_rpc_method_handler(
                    servicer.AggregatePrevotes,
                    request_deserializer=oracle_dot_v1beta1_dot_query__pb2.QueryAggregatePrevotesRequest.FromString,
                    response_serializer=oracle_dot_v1beta1_dot_query__pb2.QueryAggregatePrevotesResponse.SerializeToString,
            ),
            'AggregateVote': grpc.unary_unary_rpc_method_handler(
                    servicer.AggregateVote,
                    request_deserializer=oracle_dot_v1beta1_dot_query__pb2.QueryAggregateVoteRequest.FromString,
                    response_serializer=oracle_dot_v1beta1_dot_query__pb2.QueryAggregateVoteResponse.SerializeToString,
            ),
            'AggregateVotes': grpc.unary_unary_rpc_method_handler(
                    servicer.AggregateVotes,
                    request_deserializer=oracle_dot_v1beta1_dot_query__pb2.QueryAggregateVotesRequest.FromString,
                    response_serializer=oracle_dot_v1beta1_dot_query__pb2.QueryAggregateVotesResponse.SerializeToString,
            ),
            'Params': grpc.unary_unary_rpc_method_handler(
                    servicer.Params,
                    request_deserializer=oracle_dot_v1beta1_dot_query__pb2.QueryParamsRequest.FromString,
                    response_serializer=oracle_dot_v1beta1_dot_query__pb2.QueryParamsResponse.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'nibiru.oracle.v1beta1.Query', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class Query(object):
    """Query defines the gRPC querier service.
    """

    @staticmethod
    def ExchangeRate(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nibiru.oracle.v1beta1.Query/ExchangeRate',
            oracle_dot_v1beta1_dot_query__pb2.QueryExchangeRateRequest.SerializeToString,
            oracle_dot_v1beta1_dot_query__pb2.QueryExchangeRateResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ExchangeRates(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nibiru.oracle.v1beta1.Query/ExchangeRates',
            oracle_dot_v1beta1_dot_query__pb2.QueryExchangeRatesRequest.SerializeToString,
            oracle_dot_v1beta1_dot_query__pb2.QueryExchangeRatesResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def TobinTax(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nibiru.oracle.v1beta1.Query/TobinTax',
            oracle_dot_v1beta1_dot_query__pb2.QueryTobinTaxRequest.SerializeToString,
            oracle_dot_v1beta1_dot_query__pb2.QueryTobinTaxResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def TobinTaxes(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nibiru.oracle.v1beta1.Query/TobinTaxes',
            oracle_dot_v1beta1_dot_query__pb2.QueryTobinTaxesRequest.SerializeToString,
            oracle_dot_v1beta1_dot_query__pb2.QueryTobinTaxesResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def Actives(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nibiru.oracle.v1beta1.Query/Actives',
            oracle_dot_v1beta1_dot_query__pb2.QueryActivesRequest.SerializeToString,
            oracle_dot_v1beta1_dot_query__pb2.QueryActivesResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def VoteTargets(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nibiru.oracle.v1beta1.Query/VoteTargets',
            oracle_dot_v1beta1_dot_query__pb2.QueryVoteTargetsRequest.SerializeToString,
            oracle_dot_v1beta1_dot_query__pb2.QueryVoteTargetsResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def FeederDelegation(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nibiru.oracle.v1beta1.Query/FeederDelegation',
            oracle_dot_v1beta1_dot_query__pb2.QueryFeederDelegationRequest.SerializeToString,
            oracle_dot_v1beta1_dot_query__pb2.QueryFeederDelegationResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def MissCounter(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nibiru.oracle.v1beta1.Query/MissCounter',
            oracle_dot_v1beta1_dot_query__pb2.QueryMissCounterRequest.SerializeToString,
            oracle_dot_v1beta1_dot_query__pb2.QueryMissCounterResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def AggregatePrevote(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nibiru.oracle.v1beta1.Query/AggregatePrevote',
            oracle_dot_v1beta1_dot_query__pb2.QueryAggregatePrevoteRequest.SerializeToString,
            oracle_dot_v1beta1_dot_query__pb2.QueryAggregatePrevoteResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def AggregatePrevotes(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nibiru.oracle.v1beta1.Query/AggregatePrevotes',
            oracle_dot_v1beta1_dot_query__pb2.QueryAggregatePrevotesRequest.SerializeToString,
            oracle_dot_v1beta1_dot_query__pb2.QueryAggregatePrevotesResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def AggregateVote(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nibiru.oracle.v1beta1.Query/AggregateVote',
            oracle_dot_v1beta1_dot_query__pb2.QueryAggregateVoteRequest.SerializeToString,
            oracle_dot_v1beta1_dot_query__pb2.QueryAggregateVoteResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def AggregateVotes(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nibiru.oracle.v1beta1.Query/AggregateVotes',
            oracle_dot_v1beta1_dot_query__pb2.QueryAggregateVotesRequest.SerializeToString,
            oracle_dot_v1beta1_dot_query__pb2.QueryAggregateVotesResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def Params(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/nibiru.oracle.v1beta1.Query/Params',
            oracle_dot_v1beta1_dot_query__pb2.QueryParamsRequest.SerializeToString,
            oracle_dot_v1beta1_dot_query__pb2.QueryParamsResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.internal.enum_type_wrapper
import google.protobuf.message
import google.protobuf.timestamp_pb2
import tendermint.crypto.keys_pb2
import tendermint.crypto.proof_pb2
import tendermint.types.params_pb2
import tendermint.types.types_pb2
import typing
import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class _CheckTxType:
    ValueType = typing.NewType('ValueType', builtins.int)
    V: typing_extensions.TypeAlias = ValueType
class _CheckTxTypeEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_CheckTxType.ValueType], builtins.type):
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    NEW: _CheckTxType.ValueType  # 0
    RECHECK: _CheckTxType.ValueType  # 1
class CheckTxType(_CheckTxType, metaclass=_CheckTxTypeEnumTypeWrapper):
    pass

NEW: CheckTxType.ValueType  # 0
RECHECK: CheckTxType.ValueType  # 1
global___CheckTxType = CheckTxType


class _EvidenceType:
    ValueType = typing.NewType('ValueType', builtins.int)
    V: typing_extensions.TypeAlias = ValueType
class _EvidenceTypeEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_EvidenceType.ValueType], builtins.type):
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    UNKNOWN: _EvidenceType.ValueType  # 0
    DUPLICATE_VOTE: _EvidenceType.ValueType  # 1
    LIGHT_CLIENT_ATTACK: _EvidenceType.ValueType  # 2
class EvidenceType(_EvidenceType, metaclass=_EvidenceTypeEnumTypeWrapper):
    pass

UNKNOWN: EvidenceType.ValueType  # 0
DUPLICATE_VOTE: EvidenceType.ValueType  # 1
LIGHT_CLIENT_ATTACK: EvidenceType.ValueType  # 2
global___EvidenceType = EvidenceType


class Request(google.protobuf.message.Message):
    """This file is copied from http://github.com/tendermint/abci
    NOTE: When using custom types, mind the warnings.
    https://github.com/gogo/protobuf/blob/master/custom_types.md#warnings-and-issues

    ----------------------------------------
    Request types

    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    ECHO_FIELD_NUMBER: builtins.int
    FLUSH_FIELD_NUMBER: builtins.int
    INFO_FIELD_NUMBER: builtins.int
    SET_OPTION_FIELD_NUMBER: builtins.int
    INIT_CHAIN_FIELD_NUMBER: builtins.int
    QUERY_FIELD_NUMBER: builtins.int
    BEGIN_BLOCK_FIELD_NUMBER: builtins.int
    CHECK_TX_FIELD_NUMBER: builtins.int
    DELIVER_TX_FIELD_NUMBER: builtins.int
    END_BLOCK_FIELD_NUMBER: builtins.int
    COMMIT_FIELD_NUMBER: builtins.int
    LIST_SNAPSHOTS_FIELD_NUMBER: builtins.int
    OFFER_SNAPSHOT_FIELD_NUMBER: builtins.int
    LOAD_SNAPSHOT_CHUNK_FIELD_NUMBER: builtins.int
    APPLY_SNAPSHOT_CHUNK_FIELD_NUMBER: builtins.int
    @property
    def echo(self) -> global___RequestEcho: ...
    @property
    def flush(self) -> global___RequestFlush: ...
    @property
    def info(self) -> global___RequestInfo: ...
    @property
    def set_option(self) -> global___RequestSetOption: ...
    @property
    def init_chain(self) -> global___RequestInitChain: ...
    @property
    def query(self) -> global___RequestQuery: ...
    @property
    def begin_block(self) -> global___RequestBeginBlock: ...
    @property
    def check_tx(self) -> global___RequestCheckTx: ...
    @property
    def deliver_tx(self) -> global___RequestDeliverTx: ...
    @property
    def end_block(self) -> global___RequestEndBlock: ...
    @property
    def commit(self) -> global___RequestCommit: ...
    @property
    def list_snapshots(self) -> global___RequestListSnapshots: ...
    @property
    def offer_snapshot(self) -> global___RequestOfferSnapshot: ...
    @property
    def load_snapshot_chunk(self) -> global___RequestLoadSnapshotChunk: ...
    @property
    def apply_snapshot_chunk(self) -> global___RequestApplySnapshotChunk: ...
    def __init__(self,
        *,
        echo: typing.Optional[global___RequestEcho] = ...,
        flush: typing.Optional[global___RequestFlush] = ...,
        info: typing.Optional[global___RequestInfo] = ...,
        set_option: typing.Optional[global___RequestSetOption] = ...,
        init_chain: typing.Optional[global___RequestInitChain] = ...,
        query: typing.Optional[global___RequestQuery] = ...,
        begin_block: typing.Optional[global___RequestBeginBlock] = ...,
        check_tx: typing.Optional[global___RequestCheckTx] = ...,
        deliver_tx: typing.Optional[global___RequestDeliverTx] = ...,
        end_block: typing.Optional[global___RequestEndBlock] = ...,
        commit: typing.Optional[global___RequestCommit] = ...,
        list_snapshots: typing.Optional[global___RequestListSnapshots] = ...,
        offer_snapshot: typing.Optional[global___RequestOfferSnapshot] = ...,
        load_snapshot_chunk: typing.Optional[global___RequestLoadSnapshotChunk] = ...,
        apply_snapshot_chunk: typing.Optional[global___RequestApplySnapshotChunk] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["apply_snapshot_chunk",b"apply_snapshot_chunk","begin_block",b"begin_block","check_tx",b"check_tx","commit",b"commit","deliver_tx",b"deliver_tx","echo",b"echo","end_block",b"end_block","flush",b"flush","info",b"info","init_chain",b"init_chain","list_snapshots",b"list_snapshots","load_snapshot_chunk",b"load_snapshot_chunk","offer_snapshot",b"offer_snapshot","query",b"query","set_option",b"set_option","value",b"value"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["apply_snapshot_chunk",b"apply_snapshot_chunk","begin_block",b"begin_block","check_tx",b"check_tx","commit",b"commit","deliver_tx",b"deliver_tx","echo",b"echo","end_block",b"end_block","flush",b"flush","info",b"info","init_chain",b"init_chain","list_snapshots",b"list_snapshots","load_snapshot_chunk",b"load_snapshot_chunk","offer_snapshot",b"offer_snapshot","query",b"query","set_option",b"set_option","value",b"value"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal["value",b"value"]) -> typing.Optional[typing_extensions.Literal["echo","flush","info","set_option","init_chain","query","begin_block","check_tx","deliver_tx","end_block","commit","list_snapshots","offer_snapshot","load_snapshot_chunk","apply_snapshot_chunk"]]: ...
global___Request = Request

class RequestEcho(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    MESSAGE_FIELD_NUMBER: builtins.int
    message: typing.Text
    def __init__(self,
        *,
        message: typing.Text = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["message",b"message"]) -> None: ...
global___RequestEcho = RequestEcho

class RequestFlush(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    def __init__(self,
        ) -> None: ...
global___RequestFlush = RequestFlush

class RequestInfo(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    VERSION_FIELD_NUMBER: builtins.int
    BLOCK_VERSION_FIELD_NUMBER: builtins.int
    P2P_VERSION_FIELD_NUMBER: builtins.int
    version: typing.Text
    block_version: builtins.int
    p2p_version: builtins.int
    def __init__(self,
        *,
        version: typing.Text = ...,
        block_version: builtins.int = ...,
        p2p_version: builtins.int = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["block_version",b"block_version","p2p_version",b"p2p_version","version",b"version"]) -> None: ...
global___RequestInfo = RequestInfo

class RequestSetOption(google.protobuf.message.Message):
    """nondeterministic"""
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    KEY_FIELD_NUMBER: builtins.int
    VALUE_FIELD_NUMBER: builtins.int
    key: typing.Text
    value: typing.Text
    def __init__(self,
        *,
        key: typing.Text = ...,
        value: typing.Text = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["key",b"key","value",b"value"]) -> None: ...
global___RequestSetOption = RequestSetOption

class RequestInitChain(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    TIME_FIELD_NUMBER: builtins.int
    CHAIN_ID_FIELD_NUMBER: builtins.int
    CONSENSUS_PARAMS_FIELD_NUMBER: builtins.int
    VALIDATORS_FIELD_NUMBER: builtins.int
    APP_STATE_BYTES_FIELD_NUMBER: builtins.int
    INITIAL_HEIGHT_FIELD_NUMBER: builtins.int
    @property
    def time(self) -> google.protobuf.timestamp_pb2.Timestamp: ...
    chain_id: typing.Text
    @property
    def consensus_params(self) -> global___ConsensusParams: ...
    @property
    def validators(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___ValidatorUpdate]: ...
    app_state_bytes: builtins.bytes
    initial_height: builtins.int
    def __init__(self,
        *,
        time: typing.Optional[google.protobuf.timestamp_pb2.Timestamp] = ...,
        chain_id: typing.Text = ...,
        consensus_params: typing.Optional[global___ConsensusParams] = ...,
        validators: typing.Optional[typing.Iterable[global___ValidatorUpdate]] = ...,
        app_state_bytes: builtins.bytes = ...,
        initial_height: builtins.int = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["consensus_params",b"consensus_params","time",b"time"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["app_state_bytes",b"app_state_bytes","chain_id",b"chain_id","consensus_params",b"consensus_params","initial_height",b"initial_height","time",b"time","validators",b"validators"]) -> None: ...
global___RequestInitChain = RequestInitChain

class RequestQuery(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    DATA_FIELD_NUMBER: builtins.int
    PATH_FIELD_NUMBER: builtins.int
    HEIGHT_FIELD_NUMBER: builtins.int
    PROVE_FIELD_NUMBER: builtins.int
    data: builtins.bytes
    path: typing.Text
    height: builtins.int
    prove: builtins.bool
    def __init__(self,
        *,
        data: builtins.bytes = ...,
        path: typing.Text = ...,
        height: builtins.int = ...,
        prove: builtins.bool = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["data",b"data","height",b"height","path",b"path","prove",b"prove"]) -> None: ...
global___RequestQuery = RequestQuery

class RequestBeginBlock(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    HASH_FIELD_NUMBER: builtins.int
    HEADER_FIELD_NUMBER: builtins.int
    LAST_COMMIT_INFO_FIELD_NUMBER: builtins.int
    BYZANTINE_VALIDATORS_FIELD_NUMBER: builtins.int
    hash: builtins.bytes
    @property
    def header(self) -> tendermint.types.types_pb2.Header: ...
    @property
    def last_commit_info(self) -> global___LastCommitInfo: ...
    @property
    def byzantine_validators(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___Evidence]: ...
    def __init__(self,
        *,
        hash: builtins.bytes = ...,
        header: typing.Optional[tendermint.types.types_pb2.Header] = ...,
        last_commit_info: typing.Optional[global___LastCommitInfo] = ...,
        byzantine_validators: typing.Optional[typing.Iterable[global___Evidence]] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["header",b"header","last_commit_info",b"last_commit_info"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["byzantine_validators",b"byzantine_validators","hash",b"hash","header",b"header","last_commit_info",b"last_commit_info"]) -> None: ...
global___RequestBeginBlock = RequestBeginBlock

class RequestCheckTx(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    TX_FIELD_NUMBER: builtins.int
    TYPE_FIELD_NUMBER: builtins.int
    tx: builtins.bytes
    type: global___CheckTxType.ValueType
    def __init__(self,
        *,
        tx: builtins.bytes = ...,
        type: global___CheckTxType.ValueType = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["tx",b"tx","type",b"type"]) -> None: ...
global___RequestCheckTx = RequestCheckTx

class RequestDeliverTx(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    TX_FIELD_NUMBER: builtins.int
    tx: builtins.bytes
    def __init__(self,
        *,
        tx: builtins.bytes = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["tx",b"tx"]) -> None: ...
global___RequestDeliverTx = RequestDeliverTx

class RequestEndBlock(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    HEIGHT_FIELD_NUMBER: builtins.int
    height: builtins.int
    def __init__(self,
        *,
        height: builtins.int = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["height",b"height"]) -> None: ...
global___RequestEndBlock = RequestEndBlock

class RequestCommit(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    def __init__(self,
        ) -> None: ...
global___RequestCommit = RequestCommit

class RequestListSnapshots(google.protobuf.message.Message):
    """lists available snapshots"""
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    def __init__(self,
        ) -> None: ...
global___RequestListSnapshots = RequestListSnapshots

class RequestOfferSnapshot(google.protobuf.message.Message):
    """offers a snapshot to the application"""
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    SNAPSHOT_FIELD_NUMBER: builtins.int
    APP_HASH_FIELD_NUMBER: builtins.int
    @property
    def snapshot(self) -> global___Snapshot:
        """snapshot offered by peers"""
        pass
    app_hash: builtins.bytes
    """light client-verified app hash for snapshot height"""

    def __init__(self,
        *,
        snapshot: typing.Optional[global___Snapshot] = ...,
        app_hash: builtins.bytes = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["snapshot",b"snapshot"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["app_hash",b"app_hash","snapshot",b"snapshot"]) -> None: ...
global___RequestOfferSnapshot = RequestOfferSnapshot

class RequestLoadSnapshotChunk(google.protobuf.message.Message):
    """loads a snapshot chunk"""
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    HEIGHT_FIELD_NUMBER: builtins.int
    FORMAT_FIELD_NUMBER: builtins.int
    CHUNK_FIELD_NUMBER: builtins.int
    height: builtins.int
    format: builtins.int
    chunk: builtins.int
    def __init__(self,
        *,
        height: builtins.int = ...,
        format: builtins.int = ...,
        chunk: builtins.int = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["chunk",b"chunk","format",b"format","height",b"height"]) -> None: ...
global___RequestLoadSnapshotChunk = RequestLoadSnapshotChunk

class RequestApplySnapshotChunk(google.protobuf.message.Message):
    """Applies a snapshot chunk"""
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    INDEX_FIELD_NUMBER: builtins.int
    CHUNK_FIELD_NUMBER: builtins.int
    SENDER_FIELD_NUMBER: builtins.int
    index: builtins.int
    chunk: builtins.bytes
    sender: typing.Text
    def __init__(self,
        *,
        index: builtins.int = ...,
        chunk: builtins.bytes = ...,
        sender: typing.Text = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["chunk",b"chunk","index",b"index","sender",b"sender"]) -> None: ...
global___RequestApplySnapshotChunk = RequestApplySnapshotChunk

class Response(google.protobuf.message.Message):
    """----------------------------------------
    Response types

    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    EXCEPTION_FIELD_NUMBER: builtins.int
    ECHO_FIELD_NUMBER: builtins.int
    FLUSH_FIELD_NUMBER: builtins.int
    INFO_FIELD_NUMBER: builtins.int
    SET_OPTION_FIELD_NUMBER: builtins.int
    INIT_CHAIN_FIELD_NUMBER: builtins.int
    QUERY_FIELD_NUMBER: builtins.int
    BEGIN_BLOCK_FIELD_NUMBER: builtins.int
    CHECK_TX_FIELD_NUMBER: builtins.int
    DELIVER_TX_FIELD_NUMBER: builtins.int
    END_BLOCK_FIELD_NUMBER: builtins.int
    COMMIT_FIELD_NUMBER: builtins.int
    LIST_SNAPSHOTS_FIELD_NUMBER: builtins.int
    OFFER_SNAPSHOT_FIELD_NUMBER: builtins.int
    LOAD_SNAPSHOT_CHUNK_FIELD_NUMBER: builtins.int
    APPLY_SNAPSHOT_CHUNK_FIELD_NUMBER: builtins.int
    @property
    def exception(self) -> global___ResponseException: ...
    @property
    def echo(self) -> global___ResponseEcho: ...
    @property
    def flush(self) -> global___ResponseFlush: ...
    @property
    def info(self) -> global___ResponseInfo: ...
    @property
    def set_option(self) -> global___ResponseSetOption: ...
    @property
    def init_chain(self) -> global___ResponseInitChain: ...
    @property
    def query(self) -> global___ResponseQuery: ...
    @property
    def begin_block(self) -> global___ResponseBeginBlock: ...
    @property
    def check_tx(self) -> global___ResponseCheckTx: ...
    @property
    def deliver_tx(self) -> global___ResponseDeliverTx: ...
    @property
    def end_block(self) -> global___ResponseEndBlock: ...
    @property
    def commit(self) -> global___ResponseCommit: ...
    @property
    def list_snapshots(self) -> global___ResponseListSnapshots: ...
    @property
    def offer_snapshot(self) -> global___ResponseOfferSnapshot: ...
    @property
    def load_snapshot_chunk(self) -> global___ResponseLoadSnapshotChunk: ...
    @property
    def apply_snapshot_chunk(self) -> global___ResponseApplySnapshotChunk: ...
    def __init__(self,
        *,
        exception: typing.Optional[global___ResponseException] = ...,
        echo: typing.Optional[global___ResponseEcho] = ...,
        flush: typing.Optional[global___ResponseFlush] = ...,
        info: typing.Optional[global___ResponseInfo] = ...,
        set_option: typing.Optional[global___ResponseSetOption] = ...,
        init_chain: typing.Optional[global___ResponseInitChain] = ...,
        query: typing.Optional[global___ResponseQuery] = ...,
        begin_block: typing.Optional[global___ResponseBeginBlock] = ...,
        check_tx: typing.Optional[global___ResponseCheckTx] = ...,
        deliver_tx: typing.Optional[global___ResponseDeliverTx] = ...,
        end_block: typing.Optional[global___ResponseEndBlock] = ...,
        commit: typing.Optional[global___ResponseCommit] = ...,
        list_snapshots: typing.Optional[global___ResponseListSnapshots] = ...,
        offer_snapshot: typing.Optional[global___ResponseOfferSnapshot] = ...,
        load_snapshot_chunk: typing.Optional[global___ResponseLoadSnapshotChunk] = ...,
        apply_snapshot_chunk: typing.Optional[global___ResponseApplySnapshotChunk] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["apply_snapshot_chunk",b"apply_snapshot_chunk","begin_block",b"begin_block","check_tx",b"check_tx","commit",b"commit","deliver_tx",b"deliver_tx","echo",b"echo","end_block",b"end_block","exception",b"exception","flush",b"flush","info",b"info","init_chain",b"init_chain","list_snapshots",b"list_snapshots","load_snapshot_chunk",b"load_snapshot_chunk","offer_snapshot",b"offer_snapshot","query",b"query","set_option",b"set_option","value",b"value"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["apply_snapshot_chunk",b"apply_snapshot_chunk","begin_block",b"begin_block","check_tx",b"check_tx","commit",b"commit","deliver_tx",b"deliver_tx","echo",b"echo","end_block",b"end_block","exception",b"exception","flush",b"flush","info",b"info","init_chain",b"init_chain","list_snapshots",b"list_snapshots","load_snapshot_chunk",b"load_snapshot_chunk","offer_snapshot",b"offer_snapshot","query",b"query","set_option",b"set_option","value",b"value"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal["value",b"value"]) -> typing.Optional[typing_extensions.Literal["exception","echo","flush","info","set_option","init_chain","query","begin_block","check_tx","deliver_tx","end_block","commit","list_snapshots","offer_snapshot","load_snapshot_chunk","apply_snapshot_chunk"]]: ...
global___Response = Response

class ResponseException(google.protobuf.message.Message):
    """nondeterministic"""
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    ERROR_FIELD_NUMBER: builtins.int
    error: typing.Text
    def __init__(self,
        *,
        error: typing.Text = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["error",b"error"]) -> None: ...
global___ResponseException = ResponseException

class ResponseEcho(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    MESSAGE_FIELD_NUMBER: builtins.int
    message: typing.Text
    def __init__(self,
        *,
        message: typing.Text = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["message",b"message"]) -> None: ...
global___ResponseEcho = ResponseEcho

class ResponseFlush(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    def __init__(self,
        ) -> None: ...
global___ResponseFlush = ResponseFlush

class ResponseInfo(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    DATA_FIELD_NUMBER: builtins.int
    VERSION_FIELD_NUMBER: builtins.int
    APP_VERSION_FIELD_NUMBER: builtins.int
    LAST_BLOCK_HEIGHT_FIELD_NUMBER: builtins.int
    LAST_BLOCK_APP_HASH_FIELD_NUMBER: builtins.int
    data: typing.Text
    version: typing.Text
    app_version: builtins.int
    last_block_height: builtins.int
    last_block_app_hash: builtins.bytes
    def __init__(self,
        *,
        data: typing.Text = ...,
        version: typing.Text = ...,
        app_version: builtins.int = ...,
        last_block_height: builtins.int = ...,
        last_block_app_hash: builtins.bytes = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["app_version",b"app_version","data",b"data","last_block_app_hash",b"last_block_app_hash","last_block_height",b"last_block_height","version",b"version"]) -> None: ...
global___ResponseInfo = ResponseInfo

class ResponseSetOption(google.protobuf.message.Message):
    """nondeterministic"""
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    CODE_FIELD_NUMBER: builtins.int
    LOG_FIELD_NUMBER: builtins.int
    INFO_FIELD_NUMBER: builtins.int
    code: builtins.int
    log: typing.Text
    """bytes data = 2;"""

    info: typing.Text
    def __init__(self,
        *,
        code: builtins.int = ...,
        log: typing.Text = ...,
        info: typing.Text = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["code",b"code","info",b"info","log",b"log"]) -> None: ...
global___ResponseSetOption = ResponseSetOption

class ResponseInitChain(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    CONSENSUS_PARAMS_FIELD_NUMBER: builtins.int
    VALIDATORS_FIELD_NUMBER: builtins.int
    APP_HASH_FIELD_NUMBER: builtins.int
    @property
    def consensus_params(self) -> global___ConsensusParams: ...
    @property
    def validators(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___ValidatorUpdate]: ...
    app_hash: builtins.bytes
    def __init__(self,
        *,
        consensus_params: typing.Optional[global___ConsensusParams] = ...,
        validators: typing.Optional[typing.Iterable[global___ValidatorUpdate]] = ...,
        app_hash: builtins.bytes = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["consensus_params",b"consensus_params"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["app_hash",b"app_hash","consensus_params",b"consensus_params","validators",b"validators"]) -> None: ...
global___ResponseInitChain = ResponseInitChain

class ResponseQuery(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    CODE_FIELD_NUMBER: builtins.int
    LOG_FIELD_NUMBER: builtins.int
    INFO_FIELD_NUMBER: builtins.int
    INDEX_FIELD_NUMBER: builtins.int
    KEY_FIELD_NUMBER: builtins.int
    VALUE_FIELD_NUMBER: builtins.int
    PROOF_OPS_FIELD_NUMBER: builtins.int
    HEIGHT_FIELD_NUMBER: builtins.int
    CODESPACE_FIELD_NUMBER: builtins.int
    code: builtins.int
    log: typing.Text
    """bytes data = 2; // use "value" instead.
    nondeterministic
    """

    info: typing.Text
    """nondeterministic"""

    index: builtins.int
    key: builtins.bytes
    value: builtins.bytes
    @property
    def proof_ops(self) -> tendermint.crypto.proof_pb2.ProofOps: ...
    height: builtins.int
    codespace: typing.Text
    def __init__(self,
        *,
        code: builtins.int = ...,
        log: typing.Text = ...,
        info: typing.Text = ...,
        index: builtins.int = ...,
        key: builtins.bytes = ...,
        value: builtins.bytes = ...,
        proof_ops: typing.Optional[tendermint.crypto.proof_pb2.ProofOps] = ...,
        height: builtins.int = ...,
        codespace: typing.Text = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["proof_ops",b"proof_ops"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["code",b"code","codespace",b"codespace","height",b"height","index",b"index","info",b"info","key",b"key","log",b"log","proof_ops",b"proof_ops","value",b"value"]) -> None: ...
global___ResponseQuery = ResponseQuery

class ResponseBeginBlock(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    EVENTS_FIELD_NUMBER: builtins.int
    @property
    def events(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___Event]: ...
    def __init__(self,
        *,
        events: typing.Optional[typing.Iterable[global___Event]] = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["events",b"events"]) -> None: ...
global___ResponseBeginBlock = ResponseBeginBlock

class ResponseCheckTx(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    CODE_FIELD_NUMBER: builtins.int
    DATA_FIELD_NUMBER: builtins.int
    LOG_FIELD_NUMBER: builtins.int
    INFO_FIELD_NUMBER: builtins.int
    GAS_WANTED_FIELD_NUMBER: builtins.int
    GAS_USED_FIELD_NUMBER: builtins.int
    EVENTS_FIELD_NUMBER: builtins.int
    CODESPACE_FIELD_NUMBER: builtins.int
    code: builtins.int
    data: builtins.bytes
    log: typing.Text
    """nondeterministic"""

    info: typing.Text
    """nondeterministic"""

    gas_wanted: builtins.int
    gas_used: builtins.int
    @property
    def events(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___Event]: ...
    codespace: typing.Text
    def __init__(self,
        *,
        code: builtins.int = ...,
        data: builtins.bytes = ...,
        log: typing.Text = ...,
        info: typing.Text = ...,
        gas_wanted: builtins.int = ...,
        gas_used: builtins.int = ...,
        events: typing.Optional[typing.Iterable[global___Event]] = ...,
        codespace: typing.Text = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["code",b"code","codespace",b"codespace","data",b"data","events",b"events","gas_used",b"gas_used","gas_wanted",b"gas_wanted","info",b"info","log",b"log"]) -> None: ...
global___ResponseCheckTx = ResponseCheckTx

class ResponseDeliverTx(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    CODE_FIELD_NUMBER: builtins.int
    DATA_FIELD_NUMBER: builtins.int
    LOG_FIELD_NUMBER: builtins.int
    INFO_FIELD_NUMBER: builtins.int
    GAS_WANTED_FIELD_NUMBER: builtins.int
    GAS_USED_FIELD_NUMBER: builtins.int
    EVENTS_FIELD_NUMBER: builtins.int
    CODESPACE_FIELD_NUMBER: builtins.int
    code: builtins.int
    data: builtins.bytes
    log: typing.Text
    """nondeterministic"""

    info: typing.Text
    """nondeterministic"""

    gas_wanted: builtins.int
    gas_used: builtins.int
    @property
    def events(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___Event]: ...
    codespace: typing.Text
    def __init__(self,
        *,
        code: builtins.int = ...,
        data: builtins.bytes = ...,
        log: typing.Text = ...,
        info: typing.Text = ...,
        gas_wanted: builtins.int = ...,
        gas_used: builtins.int = ...,
        events: typing.Optional[typing.Iterable[global___Event]] = ...,
        codespace: typing.Text = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["code",b"code","codespace",b"codespace","data",b"data","events",b"events","gas_used",b"gas_used","gas_wanted",b"gas_wanted","info",b"info","log",b"log"]) -> None: ...
global___ResponseDeliverTx = ResponseDeliverTx

class ResponseEndBlock(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    VALIDATOR_UPDATES_FIELD_NUMBER: builtins.int
    CONSENSUS_PARAM_UPDATES_FIELD_NUMBER: builtins.int
    EVENTS_FIELD_NUMBER: builtins.int
    @property
    def validator_updates(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___ValidatorUpdate]: ...
    @property
    def consensus_param_updates(self) -> global___ConsensusParams: ...
    @property
    def events(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___Event]: ...
    def __init__(self,
        *,
        validator_updates: typing.Optional[typing.Iterable[global___ValidatorUpdate]] = ...,
        consensus_param_updates: typing.Optional[global___ConsensusParams] = ...,
        events: typing.Optional[typing.Iterable[global___Event]] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["consensus_param_updates",b"consensus_param_updates"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["consensus_param_updates",b"consensus_param_updates","events",b"events","validator_updates",b"validator_updates"]) -> None: ...
global___ResponseEndBlock = ResponseEndBlock

class ResponseCommit(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    DATA_FIELD_NUMBER: builtins.int
    RETAIN_HEIGHT_FIELD_NUMBER: builtins.int
    data: builtins.bytes
    """reserve 1"""

    retain_height: builtins.int
    def __init__(self,
        *,
        data: builtins.bytes = ...,
        retain_height: builtins.int = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["data",b"data","retain_height",b"retain_height"]) -> None: ...
global___ResponseCommit = ResponseCommit

class ResponseListSnapshots(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    SNAPSHOTS_FIELD_NUMBER: builtins.int
    @property
    def snapshots(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___Snapshot]: ...
    def __init__(self,
        *,
        snapshots: typing.Optional[typing.Iterable[global___Snapshot]] = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["snapshots",b"snapshots"]) -> None: ...
global___ResponseListSnapshots = ResponseListSnapshots

class ResponseOfferSnapshot(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    class _Result:
        ValueType = typing.NewType('ValueType', builtins.int)
        V: typing_extensions.TypeAlias = ValueType
    class _ResultEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[ResponseOfferSnapshot._Result.ValueType], builtins.type):
        DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
        UNKNOWN: ResponseOfferSnapshot._Result.ValueType  # 0
        """Unknown result, abort all snapshot restoration"""

        ACCEPT: ResponseOfferSnapshot._Result.ValueType  # 1
        """Snapshot accepted, apply chunks"""

        ABORT: ResponseOfferSnapshot._Result.ValueType  # 2
        """Abort all snapshot restoration"""

        REJECT: ResponseOfferSnapshot._Result.ValueType  # 3
        """Reject this specific snapshot, try others"""

        REJECT_FORMAT: ResponseOfferSnapshot._Result.ValueType  # 4
        """Reject all snapshots of this format, try others"""

        REJECT_SENDER: ResponseOfferSnapshot._Result.ValueType  # 5
        """Reject all snapshots from the sender(s), try others"""

    class Result(_Result, metaclass=_ResultEnumTypeWrapper):
        pass

    UNKNOWN: ResponseOfferSnapshot.Result.ValueType  # 0
    """Unknown result, abort all snapshot restoration"""

    ACCEPT: ResponseOfferSnapshot.Result.ValueType  # 1
    """Snapshot accepted, apply chunks"""

    ABORT: ResponseOfferSnapshot.Result.ValueType  # 2
    """Abort all snapshot restoration"""

    REJECT: ResponseOfferSnapshot.Result.ValueType  # 3
    """Reject this specific snapshot, try others"""

    REJECT_FORMAT: ResponseOfferSnapshot.Result.ValueType  # 4
    """Reject all snapshots of this format, try others"""

    REJECT_SENDER: ResponseOfferSnapshot.Result.ValueType  # 5
    """Reject all snapshots from the sender(s), try others"""


    RESULT_FIELD_NUMBER: builtins.int
    result: global___ResponseOfferSnapshot.Result.ValueType
    def __init__(self,
        *,
        result: global___ResponseOfferSnapshot.Result.ValueType = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["result",b"result"]) -> None: ...
global___ResponseOfferSnapshot = ResponseOfferSnapshot

class ResponseLoadSnapshotChunk(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    CHUNK_FIELD_NUMBER: builtins.int
    chunk: builtins.bytes
    def __init__(self,
        *,
        chunk: builtins.bytes = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["chunk",b"chunk"]) -> None: ...
global___ResponseLoadSnapshotChunk = ResponseLoadSnapshotChunk

class ResponseApplySnapshotChunk(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    class _Result:
        ValueType = typing.NewType('ValueType', builtins.int)
        V: typing_extensions.TypeAlias = ValueType
    class _ResultEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[ResponseApplySnapshotChunk._Result.ValueType], builtins.type):
        DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
        UNKNOWN: ResponseApplySnapshotChunk._Result.ValueType  # 0
        """Unknown result, abort all snapshot restoration"""

        ACCEPT: ResponseApplySnapshotChunk._Result.ValueType  # 1
        """Chunk successfully accepted"""

        ABORT: ResponseApplySnapshotChunk._Result.ValueType  # 2
        """Abort all snapshot restoration"""

        RETRY: ResponseApplySnapshotChunk._Result.ValueType  # 3
        """Retry chunk (combine with refetch and reject)"""

        RETRY_SNAPSHOT: ResponseApplySnapshotChunk._Result.ValueType  # 4
        """Retry snapshot (combine with refetch and reject)"""

        REJECT_SNAPSHOT: ResponseApplySnapshotChunk._Result.ValueType  # 5
        """Reject this snapshot, try others"""

    class Result(_Result, metaclass=_ResultEnumTypeWrapper):
        pass

    UNKNOWN: ResponseApplySnapshotChunk.Result.ValueType  # 0
    """Unknown result, abort all snapshot restoration"""

    ACCEPT: ResponseApplySnapshotChunk.Result.ValueType  # 1
    """Chunk successfully accepted"""

    ABORT: ResponseApplySnapshotChunk.Result.ValueType  # 2
    """Abort all snapshot restoration"""

    RETRY: ResponseApplySnapshotChunk.Result.ValueType  # 3
    """Retry chunk (combine with refetch and reject)"""

    RETRY_SNAPSHOT: ResponseApplySnapshotChunk.Result.ValueType  # 4
    """Retry snapshot (combine with refetch and reject)"""

    REJECT_SNAPSHOT: ResponseApplySnapshotChunk.Result.ValueType  # 5
    """Reject this snapshot, try others"""


    RESULT_FIELD_NUMBER: builtins.int
    REFETCH_CHUNKS_FIELD_NUMBER: builtins.int
    REJECT_SENDERS_FIELD_NUMBER: builtins.int
    result: global___ResponseApplySnapshotChunk.Result.ValueType
    @property
    def refetch_chunks(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.int]:
        """Chunks to refetch and reapply"""
        pass
    @property
    def reject_senders(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[typing.Text]:
        """Chunk senders to reject and ban"""
        pass
    def __init__(self,
        *,
        result: global___ResponseApplySnapshotChunk.Result.ValueType = ...,
        refetch_chunks: typing.Optional[typing.Iterable[builtins.int]] = ...,
        reject_senders: typing.Optional[typing.Iterable[typing.Text]] = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["refetch_chunks",b"refetch_chunks","reject_senders",b"reject_senders","result",b"result"]) -> None: ...
global___ResponseApplySnapshotChunk = ResponseApplySnapshotChunk

class ConsensusParams(google.protobuf.message.Message):
    """----------------------------------------
    Misc.

    ConsensusParams contains all consensus-relevant parameters
    that can be adjusted by the abci app
    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    BLOCK_FIELD_NUMBER: builtins.int
    EVIDENCE_FIELD_NUMBER: builtins.int
    VALIDATOR_FIELD_NUMBER: builtins.int
    VERSION_FIELD_NUMBER: builtins.int
    @property
    def block(self) -> global___BlockParams: ...
    @property
    def evidence(self) -> tendermint.types.params_pb2.EvidenceParams: ...
    @property
    def validator(self) -> tendermint.types.params_pb2.ValidatorParams: ...
    @property
    def version(self) -> tendermint.types.params_pb2.VersionParams: ...
    def __init__(self,
        *,
        block: typing.Optional[global___BlockParams] = ...,
        evidence: typing.Optional[tendermint.types.params_pb2.EvidenceParams] = ...,
        validator: typing.Optional[tendermint.types.params_pb2.ValidatorParams] = ...,
        version: typing.Optional[tendermint.types.params_pb2.VersionParams] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["block",b"block","evidence",b"evidence","validator",b"validator","version",b"version"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["block",b"block","evidence",b"evidence","validator",b"validator","version",b"version"]) -> None: ...
global___ConsensusParams = ConsensusParams

class BlockParams(google.protobuf.message.Message):
    """BlockParams contains limits on the block size."""
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    MAX_BYTES_FIELD_NUMBER: builtins.int
    MAX_GAS_FIELD_NUMBER: builtins.int
    max_bytes: builtins.int
    """Note: must be greater than 0"""

    max_gas: builtins.int
    """Note: must be greater or equal to -1"""

    def __init__(self,
        *,
        max_bytes: builtins.int = ...,
        max_gas: builtins.int = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["max_bytes",b"max_bytes","max_gas",b"max_gas"]) -> None: ...
global___BlockParams = BlockParams

class LastCommitInfo(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    ROUND_FIELD_NUMBER: builtins.int
    VOTES_FIELD_NUMBER: builtins.int
    round: builtins.int
    @property
    def votes(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___VoteInfo]: ...
    def __init__(self,
        *,
        round: builtins.int = ...,
        votes: typing.Optional[typing.Iterable[global___VoteInfo]] = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["round",b"round","votes",b"votes"]) -> None: ...
global___LastCommitInfo = LastCommitInfo

class Event(google.protobuf.message.Message):
    """Event allows application developers to attach additional information to
    ResponseBeginBlock, ResponseEndBlock, ResponseCheckTx and ResponseDeliverTx.
    Later, transactions may be queried using these events.
    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    TYPE_FIELD_NUMBER: builtins.int
    ATTRIBUTES_FIELD_NUMBER: builtins.int
    type: typing.Text
    @property
    def attributes(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___EventAttribute]: ...
    def __init__(self,
        *,
        type: typing.Text = ...,
        attributes: typing.Optional[typing.Iterable[global___EventAttribute]] = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["attributes",b"attributes","type",b"type"]) -> None: ...
global___Event = Event

class EventAttribute(google.protobuf.message.Message):
    """EventAttribute is a single key-value pair, associated with an event."""
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    KEY_FIELD_NUMBER: builtins.int
    VALUE_FIELD_NUMBER: builtins.int
    INDEX_FIELD_NUMBER: builtins.int
    key: builtins.bytes
    value: builtins.bytes
    index: builtins.bool
    """nondeterministic"""

    def __init__(self,
        *,
        key: builtins.bytes = ...,
        value: builtins.bytes = ...,
        index: builtins.bool = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["index",b"index","key",b"key","value",b"value"]) -> None: ...
global___EventAttribute = EventAttribute

class TxResult(google.protobuf.message.Message):
    """TxResult contains results of executing the transaction.

    One usage is indexing transaction results.
    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    HEIGHT_FIELD_NUMBER: builtins.int
    INDEX_FIELD_NUMBER: builtins.int
    TX_FIELD_NUMBER: builtins.int
    RESULT_FIELD_NUMBER: builtins.int
    height: builtins.int
    index: builtins.int
    tx: builtins.bytes
    @property
    def result(self) -> global___ResponseDeliverTx: ...
    def __init__(self,
        *,
        height: builtins.int = ...,
        index: builtins.int = ...,
        tx: builtins.bytes = ...,
        result: typing.Optional[global___ResponseDeliverTx] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["result",b"result"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["height",b"height","index",b"index","result",b"result","tx",b"tx"]) -> None: ...
global___TxResult = TxResult

class Validator(google.protobuf.message.Message):
    """----------------------------------------
    Blockchain Types

    Validator
    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    ADDRESS_FIELD_NUMBER: builtins.int
    POWER_FIELD_NUMBER: builtins.int
    address: builtins.bytes
    """The first 20 bytes of SHA256(public key)"""

    power: builtins.int
    """PubKey pub_key = 2 [(gogoproto.nullable)=false];
    The voting power
    """

    def __init__(self,
        *,
        address: builtins.bytes = ...,
        power: builtins.int = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["address",b"address","power",b"power"]) -> None: ...
global___Validator = Validator

class ValidatorUpdate(google.protobuf.message.Message):
    """ValidatorUpdate"""
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    PUB_KEY_FIELD_NUMBER: builtins.int
    POWER_FIELD_NUMBER: builtins.int
    @property
    def pub_key(self) -> tendermint.crypto.keys_pb2.PublicKey: ...
    power: builtins.int
    def __init__(self,
        *,
        pub_key: typing.Optional[tendermint.crypto.keys_pb2.PublicKey] = ...,
        power: builtins.int = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["pub_key",b"pub_key"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["power",b"power","pub_key",b"pub_key"]) -> None: ...
global___ValidatorUpdate = ValidatorUpdate

class VoteInfo(google.protobuf.message.Message):
    """VoteInfo"""
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    VALIDATOR_FIELD_NUMBER: builtins.int
    SIGNED_LAST_BLOCK_FIELD_NUMBER: builtins.int
    @property
    def validator(self) -> global___Validator: ...
    signed_last_block: builtins.bool
    def __init__(self,
        *,
        validator: typing.Optional[global___Validator] = ...,
        signed_last_block: builtins.bool = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["validator",b"validator"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["signed_last_block",b"signed_last_block","validator",b"validator"]) -> None: ...
global___VoteInfo = VoteInfo

class Evidence(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    TYPE_FIELD_NUMBER: builtins.int
    VALIDATOR_FIELD_NUMBER: builtins.int
    HEIGHT_FIELD_NUMBER: builtins.int
    TIME_FIELD_NUMBER: builtins.int
    TOTAL_VOTING_POWER_FIELD_NUMBER: builtins.int
    type: global___EvidenceType.ValueType
    @property
    def validator(self) -> global___Validator:
        """The offending validator"""
        pass
    height: builtins.int
    """The height when the offense occurred"""

    @property
    def time(self) -> google.protobuf.timestamp_pb2.Timestamp:
        """The corresponding time where the offense occurred"""
        pass
    total_voting_power: builtins.int
    """Total voting power of the validator set in case the ABCI application does
    not store historical validators.
    https://github.com/tendermint/tendermint/issues/4581
    """

    def __init__(self,
        *,
        type: global___EvidenceType.ValueType = ...,
        validator: typing.Optional[global___Validator] = ...,
        height: builtins.int = ...,
        time: typing.Optional[google.protobuf.timestamp_pb2.Timestamp] = ...,
        total_voting_power: builtins.int = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["time",b"time","validator",b"validator"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["height",b"height","time",b"time","total_voting_power",b"total_voting_power","type",b"type","validator",b"validator"]) -> None: ...
global___Evidence = Evidence

class Snapshot(google.protobuf.message.Message):
    """----------------------------------------
    State Sync Types

    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    HEIGHT_FIELD_NUMBER: builtins.int
    FORMAT_FIELD_NUMBER: builtins.int
    CHUNKS_FIELD_NUMBER: builtins.int
    HASH_FIELD_NUMBER: builtins.int
    METADATA_FIELD_NUMBER: builtins.int
    height: builtins.int
    """The height at which the snapshot was taken"""

    format: builtins.int
    """The application-specific snapshot format"""

    chunks: builtins.int
    """Number of chunks in the snapshot"""

    hash: builtins.bytes
    """Arbitrary snapshot hash, equal only if identical"""

    metadata: builtins.bytes
    """Arbitrary application metadata"""

    def __init__(self,
        *,
        height: builtins.int = ...,
        format: builtins.int = ...,
        chunks: builtins.int = ...,
        hash: builtins.bytes = ...,
        metadata: builtins.bytes = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["chunks",b"chunks","format",b"format","hash",b"hash","height",b"height","metadata",b"metadata"]) -> None: ...
global___Snapshot = Snapshot



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import cosmos.base.v1beta1.coin_pb2
import google.protobuf.any_pb2
import google.protobuf.descriptor
import google.protobuf.duration_pb2
import google.protobuf.internal.containers
import google.protobuf.internal.enum_type_wrapper
import google.protobuf.message
import google.protobuf.timestamp_pb2
import typing
import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class _VoteOption:
    ValueType = typing.NewType('ValueType', builtins.int)
    V: typing_extensions.TypeAlias = ValueType
class _VoteOptionEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_VoteOption.ValueType], builtins.type):
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    VOTE_OPTION_UNSPECIFIED: _VoteOption.ValueType  # 0
    """VOTE_OPTION_UNSPECIFIED defines a no-op vote option."""

    VOTE_OPTION_YES: _VoteOption.ValueType  # 1
    """VOTE_OPTION_YES defines a yes vote option."""

    VOTE_OPTION_ABSTAIN: _VoteOption.ValueType  # 2
    """VOTE_OPTION_ABSTAIN defines an abstain vote option."""

    VOTE_OPTION_NO: _VoteOption.ValueType  # 3
    """VOTE_OPTION_NO defines a no vote option."""

    VOTE_OPTION_NO_WITH_VETO: _VoteOption.ValueType  # 4
    """VOTE_OPTION_NO_WITH_VETO defines a no with veto vote option."""

class VoteOption(_VoteOption, metaclass=_VoteOptionEnumTypeWrapper):
    """VoteOption enumerates the valid vote options for a given governance proposal."""
    pass

VOTE_OPTION_UNSPECIFIED: VoteOption.ValueType  # 0
"""VOTE_OPTION_UNSPECIFIED defines a no-op vote option."""

VOTE_OPTION_YES: VoteOption.ValueType  # 1
"""VOTE_OPTION_YES defines a yes vote option."""

VOTE_OPTION_ABSTAIN: VoteOption.ValueType  # 2
"""VOTE_OPTION_ABSTAIN defines an abstain vote option."""

VOTE_OPTION_NO: VoteOption.ValueType  # 3
"""VOTE_OPTION_NO defines a no vote option."""

VOTE_OPTION_NO_WITH_VETO: VoteOption.ValueType  # 4
"""VOTE_OPTION_NO_WITH_VETO defines a no with veto vote option."""

global___VoteOption = VoteOption


class _ProposalStatus:
    ValueType = typing.NewType('ValueType', builtins.int)
    V: typing_extensions.TypeAlias = ValueType
class _ProposalStatusEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_ProposalStatus.ValueType], builtins.type):
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    PROPOSAL_STATUS_UNSPECIFIED: _ProposalStatus.ValueType  # 0
    """PROPOSAL_STATUS_UNSPECIFIED defines the default propopsal status."""

    PROPOSAL_STATUS_DEPOSIT_PERIOD: _ProposalStatus.ValueType  # 1
    """PROPOSAL_STATUS_DEPOSIT_PERIOD defines a proposal status during the deposit
    period.
    """

    PROPOSAL_STATUS_VOTING_PERIOD: _ProposalStatus.ValueType  # 2
    """PROPOSAL_STATUS_VOTING_PERIOD defines a proposal status during the voting
    period.
    """

    PROPOSAL_STATUS_PASSED: _ProposalStatus.ValueType  # 3
    """PROPOSAL_STATUS_PASSED defines a proposal status of a proposal that has
    passed.
    """

    PROPOSAL_STATUS_REJECTED: _ProposalStatus.ValueType  # 4
    """PROPOSAL_STATUS_REJECTED defines a proposal status of a proposal that has
    been rejected.
    """

    PROPOSAL_STATUS_FAILED: _ProposalStatus.ValueType  # 5
    """PROPOSAL_STATUS_FAILED defines a proposal status of a proposal that has
    failed.
    """

class ProposalStatus(_ProposalStatus, metaclass=_ProposalStatusEnumTypeWrapper):
    """ProposalStatus enumerates the valid statuses of a proposal."""
    pass

PROPOSAL_STATUS_UNSPECIFIED: ProposalStatus.ValueType  # 0
"""PROPOSAL_STATUS_UNSPECIFIED defines the default propopsal status."""

PROPOSAL_STATUS_DEPOSIT_PERIOD: ProposalStatus.ValueType  # 1
"""PROPOSAL_STATUS_DEPOSIT_PERIOD defines a proposal status during the deposit
period.
"""

PROPOSAL_STATUS_VOTING_PERIOD: ProposalStatus.ValueType  # 2
"""PROPOSAL_STATUS_VOTING_PERIOD defines a proposal status during the voting
period.
"""

PROPOSAL_STATUS_PASSED: ProposalStatus.ValueType  # 3
"""PROPOSAL_STATUS_PASSED defines a proposal status of a proposal that has
passed.
"""

PROPOSAL_STATUS_REJECTED: ProposalStatus.ValueType  # 4
"""PROPOSAL_STATUS_REJECTED defines a proposal status of a proposal that has
been rejected.
"""

PROPOSAL_STATUS_FAILED: ProposalStatus.ValueType  # 5
"""PROPOSAL_STATUS_FAILED defines a proposal status of a proposal that has
failed.
"""

global___ProposalStatus = ProposalStatus


class WeightedVoteOption(google.protobuf.message.Message):
    """WeightedVoteOption defines a unit of vote for vote split.

    Since: cosmos-sdk 0.43
    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    OPTION_FIELD_NUMBER: builtins.int
    WEIGHT_FIELD_NUMBER: builtins.int
    option: global___VoteOption.ValueType
    weight: typing.Text
    def __init__(self,
        *,
        option: global___VoteOption.ValueType = ...,
        weight: typing.Text = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["option",b"option","weight",b"weight"]) -> None: ...
global___WeightedVoteOption = WeightedVoteOption

class TextProposal(google.protobuf.message.Message):
    """TextProposal defines a standard text proposal whose changes need to be
    manually updated in case of approval.
    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    TITLE_FIELD_NUMBER: builtins.int
    DESCRIPTION_FIELD_NUMBER: builtins.int
    title: typing.Text
    description: typing.Text
    def __init__(self,
        *,
        title: typing.Text = ...,
        description: typing.Text = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["description",b"description","title",b"title"]) -> None: ...
global___TextProposal = TextProposal

class Deposit(google.protobuf.message.Message):
    """Deposit defines an amount deposited by an account address to an active
    proposal.
    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    PROPOSAL_ID_FIELD_NUMBER: builtins.int
    DEPOSITOR_FIELD_NUMBER: builtins.int
    AMOUNT_FIELD_NUMBER: builtins.int
    proposal_id: builtins.int
    depositor: typing.Text
    @property
    def amount(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[cosmos.base.v1beta1.coin_pb2.Coin]: ...
    def __init__(self,
        *,
        proposal_id: builtins.int = ...,
        depositor: typing.Text = ...,
        amount: typing.Optional[typing.Iterable[cosmos.base.v1beta1.coin_pb2.Coin]] = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["amount",b"amount","depositor",b"depositor","proposal_id",b"proposal_id"]) -> None: ...
global___Deposit = Deposit

class Proposal(google.protobuf.message.Message):
    """Proposal defines the core field members of a governance proposal."""
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    PROPOSAL_ID_FIELD_NUMBER: builtins.int
    CONTENT_FIELD_NUMBER: builtins.int
    STATUS_FIELD_NUMBER: builtins.int
    FINAL_TALLY_RESULT_FIELD_NUMBER: builtins.int
    SUBMIT_TIME_FIELD_NUMBER: builtins.int
    DEPOSIT_END_TIME_FIELD_NUMBER: builtins.int
    TOTAL_DEPOSIT_FIELD_NUMBER: builtins.int
    VOTING_START_TIME_FIELD_NUMBER: builtins.int
    VOTING_END_TIME_FIELD_NUMBER: builtins.int
    proposal_id: builtins.int
    @property
    def content(self) -> google.protobuf.any_pb2.Any: ...
    status: global___ProposalStatus.ValueType
    @property
    def final_tally_result(self) -> global___TallyResult: ...
    @property
    def submit_time(self) -> google.protobuf.timestamp_pb2.Timestamp: ...
    @property
    def deposit_end_time(self) -> google.protobuf.timestamp_pb2.Timestamp: ...
    @property
    def total_deposit(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[cosmos.base.v1beta1.coin_pb2.Coin]: ...
    @property
    def voting_start_time(self) -> google.protobuf.timestamp_pb2.Timestamp: ...
    @property
    def voting_end_time(self) -> google.protobuf.timestamp_pb2.Timestamp: ...
    def __init__(self,
        *,
        proposal_id: builtins.int = ...,
        content: typing.Optional[google.protobuf.any_pb2.Any] = ...,
        status: global___ProposalStatus.ValueType = ...,
        final_tally_result: typing.Optional[global___TallyResult] = ...,
        submit_time: typing.Optional[google.protobuf.timestamp_pb2.Timestamp] = ...,
        deposit_end_time: typing.Optional[google.protobuf.timestamp_pb2.Timestamp] = ...,
        total_deposit: typing.Optional[typing.Iterable[cosmos.base.v1beta1.coin_pb2.Coin]] = ...,
        voting_start_time: typing.Optional[google.protobuf.timestamp_pb2.Timestamp] = ...,
        voting_end_time: typing.Optional[google.protobuf.timestamp_pb2.Timestamp] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["content",b"content","deposit_end_time",b"deposit_end_time","final_tally_result",b"final_tally_result","submit_time",b"submit_time","voting_end_time",b"voting_end_time","voting_start_time",b"voting_start_time"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["content",b"content","deposit_end_time",b"deposit_end_time","final_tally_result",b"final_tally_result","proposal_id",b"proposal_id","status",b"status","submit_time",b"submit_time","total_deposit",b"total_deposit","voting_end_time",b"voting_end_time","voting_start_time",b"voting_start_time"]) -> None: ...
global___Proposal = Proposal

class TallyResult(google.protobuf.message.Message):
    """TallyResult defines a standard tally for a governance proposal."""
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    YES_FIELD_NUMBER: builtins.int
    ABSTAIN_FIELD_NUMBER: builtins.int
    NO_FIELD_NUMBER: builtins.int
    NO_WITH_VETO_FIELD_NUMBER: builtins.int
    yes: typing.Text
    abstain: typing.Text
    no: typing.Text
    no_with_veto: typing.Text
    def __init__(self,
        *,
        yes: typing.Text = ...,
        abstain: typing.Text = ...,
        no: typing.Text = ...,
        no_with_veto: typing.Text = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["abstain",b"abstain","no",b"no","no_with_veto",b"no_with_veto","yes",b"yes"]) -> None: ...
global___TallyResult = TallyResult

class Vote(google.protobuf.message.Message):
    """Vote defines a vote on a governance proposal.
    A Vote consists of a proposal ID, the voter, and the vote option.
    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    PROPOSAL_ID_FIELD_NUMBER: builtins.int
    VOTER_FIELD_NUMBER: builtins.int
    OPTION_FIELD_NUMBER: builtins.int
    OPTIONS_FIELD_NUMBER: builtins.int
    proposal_id: builtins.int
    voter: typing.Text
    option: global___VoteOption.ValueType
    """Deprecated: Prefer to use `options` instead. This field is set in queries
    if and only if `len(options) == 1` and that option has weight 1. In all
    other cases, this field will default to VOTE_OPTION_UNSPECIFIED.
    """

    @property
    def options(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___WeightedVoteOption]:
        """Since: cosmos-sdk 0.43"""
        pass
    def __init__(self,
        *,
        proposal_id: builtins.int = ...,
        voter: typing.Text = ...,
        option: global___VoteOption.ValueType = ...,
        options: typing.Optional[typing.Iterable[global___WeightedVoteOption]] = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["option",b"option","options",b"options","proposal_id",b"proposal_id","voter",b"voter"]) -> None: ...
global___Vote = Vote

class DepositParams(google.protobuf.message.Message):
    """DepositParams defines the params for deposits on governance proposals."""
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    MIN_DEPOSIT_FIELD_NUMBER: builtins.int
    MAX_DEPOSIT_PERIOD_FIELD_NUMBER: builtins.int
    @property
    def min_deposit(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[cosmos.base.v1beta1.coin_pb2.Coin]:
        """ Minimum deposit for a proposal to enter voting period."""
        pass
    @property
    def max_deposit_period(self) -> google.protobuf.duration_pb2.Duration:
        """ Maximum period for Atom holders to deposit on a proposal. Initial value: 2
         months.
        """
        pass
    def __init__(self,
        *,
        min_deposit: typing.Optional[typing.Iterable[cosmos.base.v1beta1.coin_pb2.Coin]] = ...,
        max_deposit_period: typing.Optional[google.protobuf.duration_pb2.Duration] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["max_deposit_period",b"max_deposit_period"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["max_deposit_period",b"max_deposit_period","min_deposit",b"min_deposit"]) -> None: ...
global___DepositParams = DepositParams

class VotingParams(google.protobuf.message.Message):
    """VotingParams defines the params for voting on governance proposals."""
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    VOTING_PERIOD_FIELD_NUMBER: builtins.int
    @property
    def voting_period(self) -> google.protobuf.duration_pb2.Duration:
        """ Length of the voting period."""
        pass
    def __init__(self,
        *,
        voting_period: typing.Optional[google.protobuf.duration_pb2.Duration] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["voting_period",b"voting_period"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["voting_period",b"voting_period"]) -> None: ...
global___VotingParams = VotingParams

class TallyParams(google.protobuf.message.Message):
    """TallyParams defines the params for tallying votes on governance proposals."""
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    QUORUM_FIELD_NUMBER: builtins.int
    THRESHOLD_FIELD_NUMBER: builtins.int
    VETO_THRESHOLD_FIELD_NUMBER: builtins.int
    quorum: builtins.bytes
    """ Minimum percentage of total stake needed to vote for a result to be
     considered valid.
    """

    threshold: builtins.bytes
    """ Minimum proportion of Yes votes for proposal to pass. Default value: 0.5."""

    veto_threshold: builtins.bytes
    """ Minimum value of Veto votes to Total votes ratio for proposal to be
     vetoed. Default value: 1/3.
    """

    def __init__(self,
        *,
        quorum: builtins.bytes = ...,
        threshold: builtins.bytes = ...,
        veto_threshold: builtins.bytes = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["quorum",b"quorum","threshold",b"threshold","veto_threshold",b"veto_threshold"]) -> None: ...
global___TallyParams = TallyParams



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import google.protobuf.descriptor
import google.protobuf.message
import perp.v1.state_pb2
import typing
import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class QueryParamsRequest(google.protobuf.message.Message):
    """---------------------------------------- Params

    QueryParamsRequest is request type for the Query/Params RPC method.
    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    def __init__(self,
        ) -> None: ...
global___QueryParamsRequest = QueryParamsRequest

class QueryParamsResponse(google.protobuf.message.Message):
    """QueryParamsResponse is response type for the Query/Params RPC method."""
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    PARAMS_FIELD_NUMBER: builtins.int
    @property
    def params(self) -> perp.v1.state_pb2.Params:
        """params holds all the parameters of this module."""
        pass
    def __init__(self,
        *,
        params: typing.Optional[perp.v1.state_pb2.Params] = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["params",b"params"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["params",b"params"]) -> None: ...
global___QueryParamsResponse = QueryParamsResponse

class QueryTraderPositionRequest(google.protobuf.message.Message):
    """---------------------------------------- TraderPosition

    QueryTraderPositionRequest is the request type for the position of the 
    x/perp module account.
    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    TOKEN_PAIR_FIELD_NUMBER: builtins.int
    TRADER_FIELD_NUMBER: builtins.int
    token_pair: typing.Text
    trader: typing.Text
    def __init__(self,
        *,
        token_pair: typing.Text = ...,
        trader: typing.Text = ...,
        ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["token_pair",b"token_pair","trader",b"trader"]) -> None: ...
global___QueryTraderPositionRequest = QueryTraderPositionRequest

class QueryTraderPositionResponse(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    POSITION_FIELD_NUMBER: builtins.int
    POSITION_NOTIONAL_FIELD_NUMBER: builtins.int
    UNREALIZED_PNL_FIELD_NUMBER: builtins.int
    MARGIN_RATIO_MARK_FIELD_NUMBER: builtins.int
    MARGIN_RATIO_INDEX_FIELD_NUMBER: builtins.int
    BLOCK_NUMBER_FIELD_NUMBER: builtins.int
    @property
    def position(self) -> perp.v1.state_pb2.Position:
        """The position as it exists in the blockchain state"""
        pass
    position_notional: typing.Text
    """The position's current notional value, if it were to be entirely closed (in margin units)."""

    unrealized_pnl: typing.Text
    """The position's unrealized PnL."""

    margin_ratio_mark: typing.Text
    """margin ratio of the position based on the mark price, mark TWAP. The higher
    value of the possible margin ratios (TWAP and instantaneous) is taken to be 
    'marginRatioMark'. Calculated from margin, unrealized PnL, and position notional.
    """

    margin_ratio_index: typing.Text
    """margin ratio of the position based on the index price. Calculated from margin, 
    unrealized PnL, and position notional.
    """

    block_number: builtins.int
    """BlockNumber is current block number at the time of query."""

    def __init__(self,
        *,
        position: typing.Optional[perp.v1.state_pb2.Position] = ...,
        position_notional: typing.Text = ...,
        unrealized_pnl: typing.Text = ...,
        margin_ratio_mark: typing.Text = ...,
        margin_ratio_index: typing.Text = ...,
        block_number: builtins.int = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["position",b"position"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["block_number",b"block_number","margin_ratio_index",b"margin_ratio_index","margin_ratio_mark",b"margin_ratio_mark","position",b"position","position_notional",b"position_notional","unrealized_pnl",b"unrealized_pnl"]) -> None: ...
global___QueryTraderPositionResponse = QueryTraderPositionResponse



from nibiru.common import Side
from nibiru.proto.cosmos.base.v1beta1 import coin_pb2 as coin_pb
from nibiru.proto.perp.v1 import state_pb2 as state_pb
from nibiru.proto.perp.v1 import tx_pb2 as tx
from nibiru.utils import to_sdk_dec, to_sdk_int

from .common import Tx


class Perp(Tx):
    def remove_margin(self, sender: str, token_pair: str, margin: coin_pb.Coin, **kwargs):
        """
        Remove margin for the position (token_pair + trader)

        Args:
            sender (str): The trader address
            token_pair (str): The token pair
            margin (coin_pb.Coin): The margin to remove in a coin format

        Returns:
            str: The output of the transaction
        """
        msg = tx.MsgRemoveMargin(
            sender=sender,
            token_pair=token_pair,
            margin=margin,
        )
        return super().execute_msg(msg, **kwargs)

    def add_margin(self, sender: str, token_pair: str, margin: coin_pb.Coin, **kwargs):
        """
        Add margin for the position (token_pair + trader)

        Args:
            sender (str): The trader address
            token_pair (str): The token pair
            margin (coin_pb.Coin): The margin to add in a coin format

        Returns:
            str: The output of the transaction
        """
        msg = tx.MsgAddMargin(
            sender=sender,
            token_pair=token_pair,
            margin=margin,
        )
        return super().execute_msg(msg, **kwargs)

    def liquidate(self, sender: str, token_pair: str, trader: str, **kwargs):
        """
        Send a liquidate transaction for the specified trader. You need a whitelisted address to be able to liquidate
        other traders.

        Args:
            sender (str): The address of the account sending the transaction
            token_pair (str): The token pair
            trader (str): The trader address

        Returns:
            str: The output of the transaction
        """
        msg = tx.MsgLiquidate(
            sender=sender,
            token_pair=token_pair,
            trader=trader,
        )
        return super().execute_msg(msg, **kwargs)

    def open_position(
        self,
        sender: str,
        token_pair: str,
        side: Side,
        quote_asset_amount: float,
        leverage: float,
        base_asset_amount_limit: float,
        **kwargs,
    ):
        """
        Open a posiiton using the specified parameters.

        Args:
            sender (str): The sender address
            token_pair (str): The token pair
            side (Side): The side, either Side.BUY or Side.SELL
            quote_asset_amount (float): The quote amount you want to use to buy base
            leverage (float): The leverage you want to use, typically between 1 and 15, depending on the maintenance
                margin ratio of the pool.
            base_asset_amount_limit (float): The minimum amount of base you are willing to receive for this amount of
                quote.

        Returns:
            str: The output of the transaction
        """

        pb_side = state_pb.Side.BUY if side == Side.BUY else state_pb.SELL
        quote_asset_amount_pb = to_sdk_int(quote_asset_amount)
        base_asset_amount_limit_pb = to_sdk_int(base_asset_amount_limit)
        leverage_pb = to_sdk_dec(leverage)

        msg = tx.MsgOpenPosition(
            sender=sender,
            token_pair=token_pair,
            side=pb_side,
            quote_asset_amount=quote_asset_amount_pb,
            leverage=leverage_pb,
            base_asset_amount_limit=base_asset_amount_limit_pb,
        )
        return super().execute_msg(msg, **kwargs)

    def close_position(self, sender: str, token_pair: str, **kwargs):
        """
        Close the position.

        Args:
            sender (str): The sender address
            token_pair (str): The token pair

        Returns:
            str: The output of the transaction
        """
        msg = tx.MsgClosePosition(
            sender=sender,
            token_pair=token_pair,
        )
        return super().execute_msg(msg, **kwargs)



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import cosmos.base.v1beta1.coin_pb2
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.internal.enum_type_wrapper
import google.protobuf.message
import typing
import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class _AuthorizationType:
    ValueType = typing.NewType('ValueType', builtins.int)
    V: typing_extensions.TypeAlias = ValueType
class _AuthorizationTypeEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_AuthorizationType.ValueType], builtins.type):
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    AUTHORIZATION_TYPE_UNSPECIFIED: _AuthorizationType.ValueType  # 0
    """AUTHORIZATION_TYPE_UNSPECIFIED specifies an unknown authorization type"""

    AUTHORIZATION_TYPE_DELEGATE: _AuthorizationType.ValueType  # 1
    """AUTHORIZATION_TYPE_DELEGATE defines an authorization type for Msg/Delegate"""

    AUTHORIZATION_TYPE_UNDELEGATE: _AuthorizationType.ValueType  # 2
    """AUTHORIZATION_TYPE_UNDELEGATE defines an authorization type for Msg/Undelegate"""

    AUTHORIZATION_TYPE_REDELEGATE: _AuthorizationType.ValueType  # 3
    """AUTHORIZATION_TYPE_REDELEGATE defines an authorization type for Msg/BeginRedelegate"""

class AuthorizationType(_AuthorizationType, metaclass=_AuthorizationTypeEnumTypeWrapper):
    """AuthorizationType defines the type of staking module authorization type

    Since: cosmos-sdk 0.43
    """
    pass

AUTHORIZATION_TYPE_UNSPECIFIED: AuthorizationType.ValueType  # 0
"""AUTHORIZATION_TYPE_UNSPECIFIED specifies an unknown authorization type"""

AUTHORIZATION_TYPE_DELEGATE: AuthorizationType.ValueType  # 1
"""AUTHORIZATION_TYPE_DELEGATE defines an authorization type for Msg/Delegate"""

AUTHORIZATION_TYPE_UNDELEGATE: AuthorizationType.ValueType  # 2
"""AUTHORIZATION_TYPE_UNDELEGATE defines an authorization type for Msg/Undelegate"""

AUTHORIZATION_TYPE_REDELEGATE: AuthorizationType.ValueType  # 3
"""AUTHORIZATION_TYPE_REDELEGATE defines an authorization type for Msg/BeginRedelegate"""

global___AuthorizationType = AuthorizationType


class StakeAuthorization(google.protobuf.message.Message):
    """StakeAuthorization defines authorization for delegate/undelegate/redelegate.

    Since: cosmos-sdk 0.43
    """
    DESCRIPTOR: google.protobuf.descriptor.Descriptor
    class Validators(google.protobuf.message.Message):
        """Validators defines list of validator addresses."""
        DESCRIPTOR: google.protobuf.descriptor.Descriptor
        ADDRESS_FIELD_NUMBER: builtins.int
        @property
        def address(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[typing.Text]: ...
        def __init__(self,
            *,
            address: typing.Optional[typing.Iterable[typing.Text]] = ...,
            ) -> None: ...
        def ClearField(self, field_name: typing_extensions.Literal["address",b"address"]) -> None: ...

    MAX_TOKENS_FIELD_NUMBER: builtins.int
    ALLOW_LIST_FIELD_NUMBER: builtins.int
    DENY_LIST_FIELD_NUMBER: builtins.int
    AUTHORIZATION_TYPE_FIELD_NUMBER: builtins.int
    @property
    def max_tokens(self) -> cosmos.base.v1beta1.coin_pb2.Coin:
        """max_tokens specifies the maximum amount of tokens can be delegate to a validator. If it is
        empty, there is no spend limit and any amount of coins can be delegated.
        """
        pass
    @property
    def allow_list(self) -> global___StakeAuthorization.Validators:
        """allow_list specifies list of validator addresses to whom grantee can delegate tokens on behalf of granter's
        account.
        """
        pass
    @property
    def deny_list(self) -> global___StakeAuthorization.Validators:
        """deny_list specifies list of validator addresses to whom grantee can not delegate tokens."""
        pass
    authorization_type: global___AuthorizationType.ValueType
    """authorization_type defines one of AuthorizationType."""

    def __init__(self,
        *,
        max_tokens: typing.Optional[cosmos.base.v1beta1.coin_pb2.Coin] = ...,
        allow_list: typing.Optional[global___StakeAuthorization.Validators] = ...,
        deny_list: typing.Optional[global___StakeAuthorization.Validators] = ...,
        authorization_type: global___AuthorizationType.ValueType = ...,
        ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["allow_list",b"allow_list","deny_list",b"deny_list","max_tokens",b"max_tokens","validators",b"validators"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["allow_list",b"allow_list","authorization_type",b"authorization_type","deny_list",b"deny_list","max_tokens",b"max_tokens","validators",b"validators"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal["validators",b"validators"]) -> typing.Optional[typing_extensions.Literal["allow_list","deny_list"]]: ...
global___StakeAuthorization = StakeAuthorization



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: oracle/v1beta1/query.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from gogoproto import gogo_pb2 as gogoproto_dot_gogo__pb2
from google.api import annotations_pb2 as google_dot_api_dot_annotations__pb2
from oracle.v1beta1 import oracle_pb2 as oracle_dot_v1beta1_dot_oracle__pb2
from cosmos.base.v1beta1 import coin_pb2 as cosmos_dot_base_dot_v1beta1_dot_coin__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1aoracle/v1beta1/query.proto\x12\x15nibiru.oracle.v1beta1\x1a\x14gogoproto/gogo.proto\x1a\x1cgoogle/api/annotations.proto\x1a\x1boracle/v1beta1/oracle.proto\x1a\x1e\x63osmos/base/v1beta1/coin.proto\"2\n\x18QueryExchangeRateRequest\x12\x0c\n\x04pair\x18\x01 \x01(\t:\x08\xe8\xa0\x1f\x00\x88\xa0\x1f\x00\"b\n\x19QueryExchangeRateResponse\x12\x45\n\rexchange_rate\x18\x01 \x01(\tB.\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Dec\xc8\xde\x1f\x00\"\x1b\n\x19QueryExchangeRatesRequest\"z\n\x1aQueryExchangeRatesResponse\x12\\\n\x0e\x65xchange_rates\x18\x01 \x03(\x0b\x32(.nibiru.oracle.v1beta1.ExchangeRateTupleB\x1a\xaa\xdf\x1f\x12\x45xchangeRateTuples\xc8\xde\x1f\x00\".\n\x14QueryTobinTaxRequest\x12\x0c\n\x04pair\x18\x01 \x01(\t:\x08\xe8\xa0\x1f\x00\x88\xa0\x1f\x00\"Z\n\x15QueryTobinTaxResponse\x12\x41\n\ttobin_tax\x18\x01 \x01(\tB.\xda\xde\x1f&github.com/cosmos/cosmos-sdk/types.Dec\xc8\xde\x1f\x00\"\x18\n\x16QueryTobinTaxesRequest\"s\n\x17QueryTobinTaxesResponse\x12X\n\x0btobin_taxes\x18\x01 \x03(\x0b\x32\x1b.nibiru.oracle.v1beta1.PairB&\xf2\xde\x1f\x12yaml:\"tobin_taxes\"\xaa\xdf\x1f\x08PairList\xc8\xde\x1f\x00\"\x15\n\x13QueryActivesRequest\"\'\n\x14QueryActivesResponse\x12\x0f\n\x07\x61\x63tives\x18\x01 \x03(\t\"\x19\n\x17QueryVoteTargetsRequest\"0\n\x18QueryVoteTargetsResponse\x12\x14\n\x0cvote_targets\x18\x01 \x03(\t\"@\n\x1cQueryFeederDelegationRequest\x12\x16\n\x0evalidator_addr\x18\x01 \x01(\t:\x08\xe8\xa0\x1f\x00\x88\xa0\x1f\x00\"4\n\x1dQueryFeederDelegationResponse\x12\x13\n\x0b\x66\x65\x65\x64\x65r_addr\x18\x01 \x01(\t\";\n\x17QueryMissCounterRequest\x12\x16\n\x0evalidator_addr\x18\x01 \x01(\t:\x08\xe8\xa0\x1f\x00\x88\xa0\x1f\x00\"0\n\x18QueryMissCounterResponse\x12\x14\n\x0cmiss_counter\x18\x01 \x01(\x04\"@\n\x1cQueryAggregatePrevoteRequest\x12\x16\n\x0evalidator_addr\x18\x01 \x01(\t:\x08\xe8\xa0\x1f\x00\x88\xa0\x1f\x00\"u\n\x1dQueryAggregatePrevoteResponse\x12T\n\x11\x61ggregate_prevote\x18\x01 \x01(\x0b\x32\x33.nibiru.oracle.v1beta1.AggregateExchangeRatePrevoteB\x04\xc8\xde\x1f\x00\"\x1f\n\x1dQueryAggregatePrevotesRequest\"w\n\x1eQueryAggregatePrevotesResponse\x12U\n\x12\x61ggregate_prevotes\x18\x01 \x03(\x0b\x32\x33.nibiru.oracle.v1beta1.AggregateExchangeRatePrevoteB\x04\xc8\xde\x1f\x00\"=\n\x19QueryAggregateVoteRequest\x12\x16\n\x0evalidator_addr\x18\x01 \x01(\t:\x08\xe8\xa0\x1f\x00\x88\xa0\x1f\x00\"l\n\x1aQueryAggregateVoteResponse\x12N\n\x0e\x61ggregate_vote\x18\x01 \x01(\x0b\x32\x30.nibiru.oracle.v1beta1.AggregateExchangeRateVoteB\x04\xc8\xde\x1f\x00\"\x1c\n\x1aQueryAggregateVotesRequest\"n\n\x1bQueryAggregateVotesResponse\x12O\n\x0f\x61ggregate_votes\x18\x01 \x03(\x0b\x32\x30.nibiru.oracle.v1beta1.AggregateExchangeRateVoteB\x04\xc8\xde\x1f\x00\"\x14\n\x12QueryParamsRequest\"J\n\x13QueryParamsResponse\x12\x33\n\x06params\x18\x01 \x01(\x0b\x32\x1d.nibiru.oracle.v1beta1.ParamsB\x04\xc8\xde\x1f\x00\x32\xdc\x11\n\x05Query\x12\xac\x01\n\x0c\x45xchangeRate\x12/.nibiru.oracle.v1beta1.QueryExchangeRateRequest\x1a\x30.nibiru.oracle.v1beta1.QueryExchangeRateResponse\"9\x82\xd3\xe4\x93\x02\x33\x12\x31/nibiru/oracle/v1beta1/pairs/{pair}/exchange_rate\x12\xa9\x01\n\rExchangeRates\x12\x30.nibiru.oracle.v1beta1.QueryExchangeRatesRequest\x1a\x31.nibiru.oracle.v1beta1.QueryExchangeRatesResponse\"3\x82\xd3\xe4\x93\x02-\x12+/nibiru/oracle/v1beta1/pairs/exchange_rates\x12\x9c\x01\n\x08TobinTax\x12+.nibiru.oracle.v1beta1.QueryTobinTaxRequest\x1a,.nibiru.oracle.v1beta1.QueryTobinTaxResponse\"5\x82\xd3\xe4\x93\x02/\x12-/nibiru/oracle/v1beta1/pairs/{pair}/tobin_tax\x12\x9d\x01\n\nTobinTaxes\x12-.nibiru.oracle.v1beta1.QueryTobinTaxesRequest\x1a..nibiru.oracle.v1beta1.QueryTobinTaxesResponse\"0\x82\xd3\xe4\x93\x02*\x12(/nibiru/oracle/v1beta1/pairs/tobin_taxes\x12\x90\x01\n\x07\x41\x63tives\x12*.nibiru.oracle.v1beta1.QueryActivesRequest\x1a+.nibiru.oracle.v1beta1.QueryActivesResponse\",\x82\xd3\xe4\x93\x02&\x12$/nibiru/oracle/v1beta1/pairs/actives\x12\xa1\x01\n\x0bVoteTargets\x12..nibiru.oracle.v1beta1.QueryVoteTargetsRequest\x1a/.nibiru.oracle.v1beta1.QueryVoteTargetsResponse\"1\x82\xd3\xe4\x93\x02+\x12)/nibiru/oracle/v1beta1/pairs/vote_targets\x12\xc0\x01\n\x10\x46\x65\x65\x64\x65rDelegation\x12\x33.nibiru.oracle.v1beta1.QueryFeederDelegationRequest\x1a\x34.nibiru.oracle.v1beta1.QueryFeederDelegationResponse\"A\x82\xd3\xe4\x93\x02;\x12\x39/nibiru/oracle/v1beta1/validators/{validator_addr}/feeder\x12\xaf\x01\n\x0bMissCounter\x12..nibiru.oracle.v1beta1.QueryMissCounterRequest\x1a/.nibiru.oracle.v1beta1.QueryMissCounterResponse\"?\x82\xd3\xe4\x93\x02\x39\x12\x37/nibiru/oracle/v1beta1/validators/{validator_addr}/miss\x12\xcb\x01\n\x10\x41ggregatePrevote\x12\x33.nibiru.oracle.v1beta1.QueryAggregatePrevoteRequest\x1a\x34.nibiru.oracle.v1beta1.QueryAggregatePrevoteResponse\"L\x82\xd3\xe4\x93\x02\x46\x12\x44/nibiru/oracle/v1beta1/validators/{validator_addr}/aggregate_prevote\x12\xbe\x01\n\x11\x41ggregatePrevotes\x12\x34.nibiru.oracle.v1beta1.QueryAggregatePrevotesRequest\x1a\x35.nibiru.oracle.v1beta1.QueryAggregatePrevotesResponse\"<\x82\xd3\xe4\x93\x02\x36\x12\x34/nibiru/oracle/v1beta1/validators/aggregate_prevotes\x12\xbf\x01\n\rAggregateVote\x12\x30.nibiru.oracle.v1beta1.QueryAggregateVoteRequest\x1a\x31.nibiru.oracle.v1beta1.QueryAggregateVoteResponse\"I\x82\xd3\xe4\x93\x02\x43\x12\x41/nibiru/oracle/v1beta1/valdiators/{validator_addr}/aggregate_vote\x12\xb2\x01\n\x0e\x41ggregateVotes\x12\x31.nibiru.oracle.v1beta1.QueryAggregateVotesRequest\x1a\x32.nibiru.oracle.v1beta1.QueryAggregateVotesResponse\"9\x82\xd3\xe4\x93\x02\x33\x12\x31/nibiru/oracle/v1beta1/validators/aggregate_votes\x12\x86\x01\n\x06Params\x12).nibiru.oracle.v1beta1.QueryParamsRequest\x1a*.nibiru.oracle.v1beta1.QueryParamsResponse\"%\x82\xd3\xe4\x93\x02\x1f\x12\x1d/nibiru/oracle/v1beta1/paramsB.Z,github.com/NibiruChain/nibiru/x/oracle/typesb\x06proto3')



_QUERYEXCHANGERATEREQUEST = DESCRIPTOR.message_types_by_name['QueryExchangeRateRequest']
_QUERYEXCHANGERATERESPONSE = DESCRIPTOR.message_types_by_name['QueryExchangeRateResponse']
_QUERYEXCHANGERATESREQUEST = DESCRIPTOR.message_types_by_name['QueryExchangeRatesRequest']
_QUERYEXCHANGERATESRESPONSE = DESCRIPTOR.message_types_by_name['QueryExchangeRatesResponse']
_QUERYTOBINTAXREQUEST = DESCRIPTOR.message_types_by_name['QueryTobinTaxRequest']
_QUERYTOBINTAXRESPONSE = DESCRIPTOR.message_types_by_name['QueryTobinTaxResponse']
_QUERYTOBINTAXESREQUEST = DESCRIPTOR.message_types_by_name['QueryTobinTaxesRequest']
_QUERYTOBINTAXESRESPONSE = DESCRIPTOR.message_types_by_name['QueryTobinTaxesResponse']
_QUERYACTIVESREQUEST = DESCRIPTOR.message_types_by_name['QueryActivesRequest']
_QUERYACTIVESRESPONSE = DESCRIPTOR.message_types_by_name['QueryActivesResponse']
_QUERYVOTETARGETSREQUEST = DESCRIPTOR.message_types_by_name['QueryVoteTargetsRequest']
_QUERYVOTETARGETSRESPONSE = DESCRIPTOR.message_types_by_name['QueryVoteTargetsResponse']
_QUERYFEEDERDELEGATIONREQUEST = DESCRIPTOR.message_types_by_name['QueryFeederDelegationRequest']
_QUERYFEEDERDELEGATIONRESPONSE = DESCRIPTOR.message_types_by_name['QueryFeederDelegationResponse']
_QUERYMISSCOUNTERREQUEST = DESCRIPTOR.message_types_by_name['QueryMissCounterRequest']
_QUERYMISSCOUNTERRESPONSE = DESCRIPTOR.message_types_by_name['QueryMissCounterResponse']
_QUERYAGGREGATEPREVOTEREQUEST = DESCRIPTOR.message_types_by_name['QueryAggregatePrevoteRequest']
_QUERYAGGREGATEPREVOTERESPONSE = DESCRIPTOR.message_types_by_name['QueryAggregatePrevoteResponse']
_QUERYAGGREGATEPREVOTESREQUEST = DESCRIPTOR.message_types_by_name['QueryAggregatePrevotesRequest']
_QUERYAGGREGATEPREVOTESRESPONSE = DESCRIPTOR.message_types_by_name['QueryAggregatePrevotesResponse']
_QUERYAGGREGATEVOTEREQUEST = DESCRIPTOR.message_types_by_name['QueryAggregateVoteRequest']
_QUERYAGGREGATEVOTERESPONSE = DESCRIPTOR.message_types_by_name['QueryAggregateVoteResponse']
_QUERYAGGREGATEVOTESREQUEST = DESCRIPTOR.message_types_by_name['QueryAggregateVotesRequest']
_QUERYAGGREGATEVOTESRESPONSE = DESCRIPTOR.message_types_by_name['QueryAggregateVotesResponse']
_QUERYPARAMSREQUEST = DESCRIPTOR.message_types_by_name['QueryParamsRequest']
_QUERYPARAMSRESPONSE = DESCRIPTOR.message_types_by_name['QueryParamsResponse']
QueryExchangeRateRequest = _reflection.GeneratedProtocolMessageType('QueryExchangeRateRequest', (_message.Message,), {
  'DESCRIPTOR' : _QUERYEXCHANGERATEREQUEST,
  '__module__' : 'oracle.v1beta1.query_pb2'
  # @@protoc_insertion_point(class_scope:nibiru.oracle.v1beta1.QueryExchangeRateRequest)
  })
_sym_db.RegisterMessage(QueryExchangeRateRequest)

QueryExchangeRateResponse = _reflection.GeneratedProtocolMessageType('QueryExchangeRateResponse', (_message.Message,), {
  'DESCRIPTOR' : _QUERYEXCHANGERATERESPONSE,
  '__module__' : 'oracle.v1beta1.query_pb2'
  # @@protoc_insertion_point(class_scope:nibiru.oracle.v1beta1.QueryExchangeRateResponse)
  })
_sym_db.RegisterMessage(QueryExchangeRateResponse)

QueryExchangeRatesRequest = _reflection.GeneratedProtocolMessageType('QueryExchangeRatesRequest', (_message.Message,), {
  'DESCRIPTOR' : _QUERYEXCHANGERATESREQUEST,
  '__module__' : 'oracle.v1beta1.query_pb2'
  # @@protoc_insertion_point(class_scope:nibiru.oracle.v1beta1.QueryExchangeRatesRequest)
  })
_sym_db.RegisterMessage(QueryExchangeRatesRequest)

QueryExchangeRatesResponse = _reflection.GeneratedProtocolMessageType('QueryExchangeRatesResponse', (_message.Message,), {
  'DESCRIPTOR' : _QUERYEXCHANGERATESRESPONSE,
  '__module__' : 'oracle.v1beta1.query_pb2'
  # @@protoc_insertion_point(class_scope:nibiru.oracle.v1beta1.QueryExchangeRatesResponse)
  })
_sym_db.RegisterMessage(QueryExchangeRatesResponse)

QueryTobinTaxRequest = _reflection.GeneratedProtocolMessageType('QueryTobinTaxRequest', (_message.Message,), {
  'DESCRIPTOR' : _QUERYTOBINTAXREQUEST,
  '__module__' : 'oracle.v1beta1.query_pb2'
  # @@protoc_insertion_point(class_scope:nibiru.oracle.v1beta1.QueryTobinTaxRequest)
  })
_sym_db.RegisterMessage(QueryTobinTaxRequest)

QueryTobinTaxResponse = _reflection.GeneratedProtocolMessageType('QueryTobinTaxResponse', (_message.Message,), {
  'DESCRIPTOR' : _QUERYTOBINTAXRESPONSE,
  '__module__' : 'oracle.v1beta1.query_pb2'
  # @@protoc_insertion_point(class_scope:nibiru.oracle.v1beta1.QueryTobinTaxResponse)
  })
_sym_db.RegisterMessage(QueryTobinTaxResponse)

QueryTobinTaxesRequest = _reflection.GeneratedProtocolMessageType('QueryTobinTaxesRequest', (_message.Message,), {
  'DESCRIPTOR' : _QUERYTOBINTAXESREQUEST,
  '__module__' : 'oracle.v1beta1.query_pb2'
  # @@protoc_insertion_point(class_scope:nibiru.oracle.v1beta1.QueryTobinTaxesRequest)
  })
_sym_db.RegisterMessage(QueryTobinTaxesRequest)

QueryTobinTaxesResponse = _reflection.GeneratedProtocolMessageType('QueryTobinTaxesResponse', (_message.Message,), {
  'DESCRIPTOR' : _QUERYTOBINTAXESRESPONSE,
  '__module__' : 'oracle.v1beta1.query_pb2'
  # @@protoc_insertion_point(class_scope:nibiru.oracle.v1beta1.QueryTobinTaxesResponse)
  })
_sym_db.RegisterMessage(QueryTobinTaxesResponse)

QueryActivesRequest = _reflection.GeneratedProtocolMessageType('QueryActivesRequest', (_message.Message,), {
  'DESCRIPTOR' : _QUERYACTIVESREQUEST,
  '__module__' : 'oracle.v1beta1.query_pb2'
  # @@protoc_insertion_point(class_scope:nibiru.oracle.v1beta1.QueryActivesRequest)
  })
_sym_db.RegisterMessage(QueryActivesRequest)

QueryActivesResponse = _reflection.GeneratedProtocolMessageType('QueryActivesResponse', (_message.Message,), {
  'DESCRIPTOR' : _QUERYACTIVESRESPONSE,
  '__module__' : 'oracle.v1beta1.query_pb2'
  # @@protoc_insertion_point(class_scope:nibiru.oracle.v1beta1.QueryActivesResponse)
  })
_sym_db.RegisterMessage(QueryActivesResponse)

QueryVoteTargetsRequest = _reflection.GeneratedProtocolMessageType('QueryVoteTargetsRequest', (_message.Message,), {
  'DESCRIPTOR' : _QUERYVOTETARGETSREQUEST,
  '__module__' : 'oracle.v1beta1.query_pb2'
  # @@protoc_insertion_point(class_scope:nibiru.oracle.v1beta1.QueryVoteTargetsRequest)
  })
_sym_db.RegisterMessage(QueryVoteTargetsRequest)

QueryVoteTargetsResponse = _reflection.GeneratedProtocolMessageType('QueryVoteTargetsResponse', (_message.Message,), {
  'DESCRIPTOR' : _QUERYVOTETARGETSRESPONSE,
  '__module__' : 'oracle.v1beta1.query_pb2'
  # @@protoc_insertion_point(class_scope:nibiru.oracle.v1beta1.QueryVoteTargetsResponse)
  })
_sym_db.RegisterMessage(QueryVoteTargetsResponse)

QueryFeederDelegationRequest = _reflection.GeneratedProtocolMessageType('QueryFeederDelegationRequest', (_message.Message,), {
  'DESCRIPTOR' : _QUERYFEEDERDELEGATIONREQUEST,
  '__module__' : 'oracle.v1beta1.query_pb2'
  # @@protoc_insertion_point(class_scope:nibiru.oracle.v1beta1.QueryFeederDelegationRequest)
  })
_sym_db.RegisterMessage(QueryFeederDelegationRequest)

QueryFeederDelegationResponse = _reflection.GeneratedProtocolMessageType('QueryFeederDelegationResponse', (_message.Message,), {
  'DESCRIPTOR' : _QUERYFEEDERDELEGATIONRESPONSE,
  '__module__' : 'oracle.v1beta1.query_pb2'
  # @@protoc_insertion_point(class_scope:nibiru.oracle.v1beta1.QueryFeederDelegationResponse)
  })
_sym_db.RegisterMessage(QueryFeederDelegationResponse)

QueryMissCounterRequest = _reflection.GeneratedProtocolMessageType('QueryMissCounterRequest', (_message.Message,), {
  'DESCRIPTOR' : _QUERYMISSCOUNTERREQUEST,
  '__module__' : 'oracle.v1beta1.query_pb2'
  # @@protoc_insertion_point(class_scope:nibiru.oracle.v1beta1.QueryMissCounterRequest)
  })
_sym_db.RegisterMessage(QueryMissCounterRequest)

QueryMissCounterResponse = _reflection.GeneratedProtocolMessageType('QueryMissCounterResponse', (_message.Message,), {
  'DESCRIPTOR' : _QUERYMISSCOUNTERRESPONSE,
  '__module__' : 'oracle.v1beta1.query_pb2'
  # @@protoc_insertion_point(class_scope:nibiru.oracle.v1beta1.QueryMissCounterResponse)
  })
_sym_db.RegisterMessage(QueryMissCounterResponse)

QueryAggregatePrevoteRequest = _reflection.GeneratedProtocolMessageType('QueryAggregatePrevoteRequest', (_message.Message,), {
  'DESCRIPTOR' : _QUERYAGGREGATEPREVOTEREQUEST,
  '__module__' : 'oracle.v1beta1.query_pb2'
  # @@protoc_insertion_point(class_scope:nibiru.oracle.v1beta1.QueryAggregatePrevoteRequest)
  })
_sym_db.RegisterMessage(QueryAggregatePrevoteRequest)

QueryAggregatePrevoteResponse = _reflection.GeneratedProtocolMessageType('QueryAggregatePrevoteResponse', (_message.Message,), {
  'DESCRIPTOR' : _QUERYAGGREGATEPREVOTERESPONSE,
  '__module__' : 'oracle.v1beta1.query_pb2'
  # @@protoc_insertion_point(class_scope:nibiru.oracle.v1beta1.QueryAggregatePrevoteResponse)
  })
_sym_db.RegisterMessage(QueryAggregatePrevoteResponse)

QueryAggregatePrevotesRequest = _reflection.GeneratedProtocolMessageType('QueryAggregatePrevotesRequest', (_message.Message,), {
  'DESCRIPTOR' : _QUERYAGGREGATEPREVOTESREQUEST,
  '__module__' : 'oracle.v1beta1.query_pb2'
  # @@protoc_insertion_point(class_scope:nibiru.oracle.v1beta1.QueryAggregatePrevotesRequest)
  })
_sym_db.RegisterMessage(QueryAggregatePrevotesRequest)

QueryAggregatePrevotesResponse = _reflection.GeneratedProtocolMessageType('QueryAggregatePrevotesResponse', (_message.Message,), {
  'DESCRIPTOR' : _QUERYAGGREGATEPREVOTESRESPONSE,
  '__module__' : 'oracle.v1beta1.query_pb2'
  # @@protoc_insertion_point(class_scope:nibiru.oracle.v1beta1.QueryAggregatePrevotesResponse)
  })
_sym_db.RegisterMessage(QueryAggregatePrevotesResponse)

QueryAggregateVoteRequest = _reflection.GeneratedProtocolMessageType('QueryAggregateVoteRequest', (_message.Message,), {
  'DESCRIPTOR' : _QUERYAGGREGATEVOTEREQUEST,
  '__module__' : 'oracle.v1beta1.query_pb2'
  # @@protoc_insertion_point(class_scope:nibiru.oracle.v1beta1.QueryAggregateVoteRequest)
  })
_sym_db.RegisterMessage(QueryAggregateVoteRequest)

QueryAggregateVoteResponse = _reflection.GeneratedProtocolMessageType('QueryAggregateVoteResponse', (_message.Message,), {
  'DESCRIPTOR' : _QUERYAGGREGATEVOTERESPONSE,
  '__module__' : 'oracle.v1beta1.query_pb2'
  # @@protoc_insertion_point(class_scope:nibiru.oracle.v1beta1.QueryAggregateVoteResponse)
  })
_sym_db.RegisterMessage(QueryAggregateVoteResponse)

QueryAggregateVotesRequest = _reflection.GeneratedProtocolMessageType('QueryAggregateVotesRequest', (_message.Message,), {
  'DESCRIPTOR' : _QUERYAGGREGATEVOTESREQUEST,
  '__module__' : 'oracle.v1beta1.query_pb2'
  # @@protoc_insertion_point(class_scope:nibiru.oracle.v1beta1.QueryAggregateVotesRequest)
  })
_sym_db.RegisterMessage(QueryAggregateVotesRequest)

QueryAggregateVotesResponse = _reflection.GeneratedProtocolMessageType('QueryAggregateVotesResponse', (_message.Message,), {
  'DESCRIPTOR' : _QUERYAGGREGATEVOTESRESPONSE,
  '__module__' : 'oracle.v1beta1.query_pb2'
  # @@protoc_insertion_point(class_scope:nibiru.oracle.v1beta1.QueryAggregateVotesResponse)
  })
_sym_db.RegisterMessage(QueryAggregateVotesResponse)

QueryParamsRequest = _reflection.GeneratedProtocolMessageType('QueryParamsRequest', (_message.Message,), {
  'DESCRIPTOR' : _QUERYPARAMSREQUEST,
  '__module__' : 'oracle.v1beta1.query_pb2'
  # @@protoc_insertion_point(class_scope:nibiru.oracle.v1beta1.QueryParamsRequest)
  })
_sym_db.RegisterMessage(QueryParamsRequest)

QueryParamsResponse = _reflection.GeneratedProtocolMessageType('QueryParamsResponse', (_message.Message,), {
  'DESCRIPTOR' : _QUERYPARAMSRESPONSE,
  '__module__' : 'oracle.v1beta1.query_pb2'
  # @@protoc_insertion_point(class_scope:nibiru.oracle.v1beta1.QueryParamsResponse)
  })
_sym_db.RegisterMessage(QueryParamsResponse)

_QUERY = DESCRIPTOR.services_by_name['Query']
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'Z,github.com/NibiruChain/nibiru/x/oracle/types'
  _QUERYEXCHANGERATEREQUEST._options = None
  _QUERYEXCHANGERATEREQUEST._serialized_options = b'\350\240\037\000\210\240\037\000'
  _QUERYEXCHANGERATERESPONSE.fields_by_name['exchange_rate']._options = None
  _QUERYEXCHANGERATERESPONSE.fields_by_name['exchange_rate']._serialized_options = b'\332\336\037&github.com/cosmos/cosmos-sdk/types.Dec\310\336\037\000'
  _QUERYEXCHANGERATESRESPONSE.fields_by_name['exchange_rates']._options = None
  _QUERYEXCHANGERATESRESPONSE.fields_by_name['exchange_rates']._serialized_options = b'\252\337\037\022ExchangeRateTuples\310\336\037\000'
  _QUERYTOBINTAXREQUEST._options = None
  _QUERYTOBINTAXREQUEST._serialized_options = b'\350\240\037\000\210\240\037\000'
  _QUERYTOBINTAXRESPONSE.fields_by_name['tobin_tax']._options = None
  _QUERYTOBINTAXRESPONSE.fields_by_name['tobin_tax']._serialized_options = b'\332\336\037&github.com/cosmos/cosmos-sdk/types.Dec\310\336\037\000'
  _QUERYTOBINTAXESRESPONSE.fields_by_name['tobin_taxes']._options = None
  _QUERYTOBINTAXESRESPONSE.fields_by_name['tobin_taxes']._serialized_options = b'\362\336\037\022yaml:\"tobin_taxes\"\252\337\037\010PairList\310\336\037\000'
  _QUERYFEEDERDELEGATIONREQUEST._options = None
  _QUERYFEEDERDELEGATIONREQUEST._serialized_options = b'\350\240\037\000\210\240\037\000'
  _QUERYMISSCOUNTERREQUEST._options = None
  _QUERYMISSCOUNTERREQUEST._serialized_options = b'\350\240\037\000\210\240\037\000'
  _QUERYAGGREGATEPREVOTEREQUEST._options = None
  _QUERYAGGREGATEPREVOTEREQUEST._serialized_options = b'\350\240\037\000\210\240\037\000'
  _QUERYAGGREGATEPREVOTERESPONSE.fields_by_name['aggregate_prevote']._options = None
  _QUERYAGGREGATEPREVOTERESPONSE.fields_by_name['aggregate_prevote']._serialized_options = b'\310\336\037\000'
  _QUERYAGGREGATEPREVOTESRESPONSE.fields_by_name['aggregate_prevotes']._options = None
  _QUERYAGGREGATEPREVOTESRESPONSE.fields_by_name['aggregate_prevotes']._serialized_options = b'\310\336\037\000'
  _QUERYAGGREGATEVOTEREQUEST._options = None
  _QUERYAGGREGATEVOTEREQUEST._serialized_options = b'\350\240\037\000\210\240\037\000'
  _QUERYAGGREGATEVOTERESPONSE.fields_by_name['aggregate_vote']._options = None
  _QUERYAGGREGATEVOTERESPONSE.fields_by_name['aggregate_vote']._serialized_options = b'\310\336\037\000'
  _QUERYAGGREGATEVOTESRESPONSE.fields_by_name['aggregate_votes']._options = None
  _QUERYAGGREGATEVOTESRESPONSE.fields_by_name['aggregate_votes']._serialized_options = b'\310\336\037\000'
  _QUERYPARAMSRESPONSE.fields_by_name['params']._options = None
  _QUERYPARAMSRESPONSE.fields_by_name['params']._serialized_options = b'\310\336\037\000'
  _QUERY.methods_by_name['ExchangeRate']._options = None
  _QUERY.methods_by_name['ExchangeRate']._serialized_options = b'\202\323\344\223\0023\0221/nibiru/oracle/v1beta1/pairs/{pair}/exchange_rate'
  _QUERY.methods_by_name['ExchangeRates']._options = None
  _QUERY.methods_by_name['ExchangeRates']._serialized_options = b'\202\323\344\223\002-\022+/nibiru/oracle/v1beta1/pairs/exchange_rates'
  _QUERY.methods_by_name['TobinTax']._options = None
  _QUERY.methods_by_name['TobinTax']._serialized_options = b'\202\323\344\223\002/\022-/nibiru/oracle/v1beta1/pairs/{pair}/tobin_tax'
  _QUERY.methods_by_name['TobinTaxes']._options = None
  _QUERY.methods_by_name['TobinTaxes']._serialized_options = b'\202\323\344\223\002*\022(/nibiru/oracle/v1beta1/pairs/tobin_taxes'
  _QUERY.methods_by_name['Actives']._options = None
  _QUERY.methods_by_name['Actives']._serialized_options = b'\202\323\344\223\002&\022$/nibiru/oracle/v1beta1/pairs/actives'
  _QUERY.methods_by_name['VoteTargets']._options = None
  _QUERY.methods_by_name['VoteTargets']._serialized_options = b'\202\323\344\223\002+\022)/nibiru/oracle/v1beta1/pairs/vote_targets'
  _QUERY.methods_by_name['FeederDelegation']._options = None
  _QUERY.methods_by_name['FeederDelegation']._serialized_options = b'\202\323\344\223\002;\0229/nibiru/oracle/v1beta1/validators/{validator_addr}/feeder'
  _QUERY.methods_by_name['MissCounter']._options = None
  _QUERY.methods_by_name['MissCounter']._serialized_options = b'\202\323\344\223\0029\0227/nibiru/oracle/v1beta1/validators/{validator_addr}/miss'
  _QUERY.methods_by_name['AggregatePrevote']._options = None
  _QUERY.methods_by_name['AggregatePrevote']._serialized_options = b'\202\323\344\223\002F\022D/nibiru/oracle/v1beta1/validators/{validator_addr}/aggregate_prevote'
  _QUERY.methods_by_name['AggregatePrevotes']._options = None
  _QUERY.methods_by_name['AggregatePrevotes']._serialized_options = b'\202\323\344\223\0026\0224/nibiru/oracle/v1beta1/validators/aggregate_prevotes'
  _QUERY.methods_by_name['AggregateVote']._options = None
  _QUERY.methods_by_name['AggregateVote']._serialized_options = b'\202\323\344\223\002C\022A/nibiru/oracle/v1beta1/valdiators/{validator_addr}/aggregate_vote'
  _QUERY.methods_by_name['AggregateVotes']._options = None
  _QUERY.methods_by_name['AggregateVotes']._serialized_options = b'\202\323\344\223\0023\0221/nibiru/oracle/v1beta1/validators/aggregate_votes'
  _QUERY.methods_by_name['Params']._options = None
  _QUERY.methods_by_name['Params']._serialized_options = b'\202\323\344\223\002\037\022\035/nibiru/oracle/v1beta1/params'
  _QUERYEXCHANGERATEREQUEST._serialized_start=166
  _QUERYEXCHANGERATEREQUEST._serialized_end=216
  _QUERYEXCHANGERATERESPONSE._serialized_start=218
  _QUERYEXCHANGERATERESPONSE._serialized_end=316
  _QUERYEXCHANGERATESREQUEST._serialized_start=318
  _QUERYEXCHANGERATESREQUEST._serialized_end=345
  _QUERYEXCHANGERATESRESPONSE._serialized_start=347
  _QUERYEXCHANGERATESRESPONSE._serialized_end=469
  _QUERYTOBINTAXREQUEST._serialized_start=471
  _QUERYTOBINTAXREQUEST._serialized_end=517
  _QUERYTOBINTAXRESPONSE._serialized_start=519
  _QUERYTOBINTAXRESPONSE._serialized_end=609
  _QUERYTOBINTAXESREQUEST._serialized_start=611
  _QUERYTOBINTAXESREQUEST._serialized_end=635
  _QUERYTOBINTAXESRESPONSE._serialized_start=637
  _QUERYTOBINTAXESRESPONSE._serialized_end=752
  _QUERYACTIVESREQUEST._serialized_start=754
  _QUERYACTIVESREQUEST._serialized_end=775
  _QUERYACTIVESRESPONSE._serialized_start=777
  _QUERYACTIVESRESPONSE._serialized_end=816
  _QUERYVOTETARGETSREQUEST._serialized_start=818
  _QUERYVOTETARGETSREQUEST._serialized_end=843
  _QUERYVOTETARGETSRESPONSE._serialized_start=845
  _QUERYVOTETARGETSRESPONSE._serialized_end=893
  _QUERYFEEDERDELEGATIONREQUEST._serialized_start=895
  _QUERYFEEDERDELEGATIONREQUEST._serialized_end=959
  _QUERYFEEDERDELEGATIONRESPONSE._serialized_start=961
  _QUERYFEEDERDELEGATIONRESPONSE._serialized_end=1013
  _QUERYMISSCOUNTERREQUEST._serialized_start=1015
  _QUERYMISSCOUNTERREQUEST._serialized_end=1074
  _QUERYMISSCOUNTERRESPONSE._serialized_start=1076
  _QUERYMISSCOUNTERRESPONSE._serialized_end=1124
  _QUERYAGGREGATEPREVOTEREQUEST._serialized_start=1126
  _QUERYAGGREGATEPREVOTEREQUEST._serialized_end=1190
  _QUERYAGGREGATEPREVOTERESPONSE._serialized_start=1192
  _QUERYAGGREGATEPREVOTERESPONSE._serialized_end=1309
  _QUERYAGGREGATEPREVOTESREQUEST._serialized_start=1311
  _QUERYAGGREGATEPREVOTESREQUEST._serialized_end=1342
  _QUERYAGGREGATEPREVOTESRESPONSE._serialized_start=1344
  _QUERYAGGREGATEPREVOTESRESPONSE._serialized_end=1463
  _QUERYAGGREGATEVOTEREQUEST._serialized_start=1465
  _QUERYAGGREGATEVOTEREQUEST._serialized_end=1526
  _QUERYAGGREGATEVOTERESPONSE._serialized_start=1528
  _QUERYAGGREGATEVOTERESPONSE._serialized_end=1636
  _QUERYAGGREGATEVOTESREQUEST._serialized_start=1638
  _QUERYAGGREGATEVOTESREQUEST._serialized_end=1666
  _QUERYAGGREGATEVOTESRESPONSE._serialized_start=1668
  _QUERYAGGREGATEVOTESRESPONSE._serialized_end=1778
  _QUERYPARAMSREQUEST._serialized_start=1780
  _QUERYPARAMSREQUEST._serialized_end=1800
  _QUERYPARAMSRESPONSE._serialized_start=1802
  _QUERYPARAMSRESPONSE._serialized_end=1876
  _QUERY._serialized_start=1879
  _QUERY._serialized_end=4147
# @@protoc_insertion_point(module_scope)
